/*
 * 	periodic_tim6_irq.c
 *
 */
__attribute__((naked)) __attribute__((section (".start_section")) )
void startup ( void )
{
__asm__ volatile(" LDR R0,=0x2001C000\n");		/* set stack */
__asm__ volatile(" MOV SP,R0\n");
__asm__ volatile(" BL main\n");					/* call main */
__asm__ volatile(".L1: B .L1\n");				/* never return */
}

/* Port D */
#define		PORT_D_BASE		0x40020C00
#define		GPIO_D_MODER 	((volatile unsigned int *)	(PORT_D_BASE))  
#define		GPIO_D_OTYPER 	((volatile unsigned short *)(PORT_D_BASE+0x4))  
#define		GPIO_D_OSPEEDR 	((volatile unsigned int *)	(PORT_D_BASE+0x8))  
#define		GPIO_D_PUPDR 	((volatile unsigned int *)	(PORT_D_BASE+0xC))  
#define		GPIO_D_IDRLOW 	((volatile unsigned char *)(PORT_D_BASE+0x10))  
#define		GPIO_D_IDRHIGH 	((volatile unsigned char *)(PORT_D_BASE+0x11))  
#define		GPIO_D_ODRLOW 	((volatile unsigned char *)	(PORT_D_BASE+0x14))  
#define		GPIO_D_ODRHIGH 	((volatile unsigned char *)	(PORT_D_BASE+0x15))  

/* Reset and Clock Generator */
#define		RCC_APB1ENR 	((volatile unsigned int *)	0x40023840)  
#define		RCC_APB1RSTR 	((volatile unsigned int *)	0x40023820)  


/* Timer 6 */
#define		TIM6_CR1 	((volatile unsigned short *)	0x40001000)  
#define		TIM6_CR2 	((volatile unsigned short *)	0x40001004) 
#define		TIM6_DIER 	((volatile unsigned short *)	0x4000100C)  
#define		TIM6_SR 	((volatile unsigned short *)	0x40001010)  
#define		TIM6_EGR 	((volatile unsigned short *)	0x40001014) 
#define		TIM6_CNT 	((volatile unsigned short *)	0x40001024)  
#define		TIM6_PSC	((volatile unsigned short *)	0x40001028)  
#define		TIM6_ARR	((volatile unsigned short *)	0x4000102C)  

#define NVIC_TIM6_IRQ_BPOS  (1<<22)
#define	NVIC_TIM6_ISER 		((volatile unsigned int *)	0xE000E104)  
#define	TIM6_IRQVEC 		((void (**)(void) ) 0x2001C118 ) 


#define UIF		(1<<0)
#define UIE		(1<<0)
#define UG		(1<<0)
#define ARPE	(1<<7)
#define OPM		(1<<3)
#define URS		(1<<2)
#define UDIS	(1<<1)
#define	CEN		(1<<0)


volatile char flag;
volatile int ms;


#define SIMULATOR

void timer6_interrupt( void )
{  /* 1 ms period */
	*TIM6_SR &= ~UIF;
	flag ^= (flag | 1); /* toggle bit 0 */

#ifdef SIMULATOR
	if( ms >= 10)
#else
	if( ms >= 1000)
#endif
	{
		flag ^= (flag | 2); /* toggle bit 1 */
		ms = 0;
	}else ms++;
}


void timer6_init( )
{
	/* System core clock: 168MHz
	 * TIM6 and TIM7 clocks: 84 MHz
	 * APB2 Clock: 84 MHz
	 * APB1 Clock: 42 MHz */
	*TIM6_CR1 &= ~CEN;	 	 
	*TIM6_IRQVEC = timer6_interrupt;
	*NVIC_TIM6_ISER |= NVIC_TIM6_IRQ_BPOS;	/* Enable this interrupt */ 
	 
	 /* freq: PSC is # of microsecs-1 */
	*TIM6_PSC= 83; 
	*TIM6_ARR = 999;
	*TIM6_DIER |= UIE;
	*TIM6_CR1 |= CEN;
}


void gpio_init( void )
{ /* for test */
	*GPIO_D_MODER = 0x00005555;		
	*GPIO_D_PUPDR = 0x00000000;		/* Input, pull down */
    *GPIO_D_OTYPER= 0x00000000;	    /* outputs are push/pull */
	*GPIO_D_ODRLOW = 0;
}

void main(void)
{
	flag = 0;
	ms = 0;
	gpio_init();
	timer6_init();
	
	while( 1 )
	{
		*GPIO_D_ODRLOW = flag;
	}
}

