/*
 * 	usart_rxtx_irq.c
 *  Observera att denna fil innehåller flera olika exempel
 */
void startup(void) __attribute__((naked)) __attribute__((section (".start_section")) );

void startup ( void )
{
__asm volatile(
	" LDR R0,=0x2001C000\n"		/* set stack */
	" MOV SP,R0\n"
	" BL main\n"				/* call main */
	" B .\n"					/* never return */
	) ;
}

typedef struct tag_usart
{
	volatile unsigned short sr;
	volatile unsigned short Unused0;
	volatile unsigned short dr;
	volatile unsigned short Unused1;
	volatile unsigned short brr;
	volatile unsigned short Unused2;
	volatile unsigned short cr1;
	volatile unsigned short Unused3;
	volatile unsigned short cr2;
	volatile unsigned short Unused4;
	volatile unsigned short cr3;
	volatile unsigned short Unused5;
	volatile unsigned short gtpr;
} USART;

#define USART1  ((USART *) 0x40011000)

#define	USART1_IRQVEC 	((void (**)(void) ) 0x2001C0D4 ) 

#define	EN		(1<<13)
#define	TXEIE	(1<<7)
#define	TCIE	(1<<6)
#define	RXNEIE	(1<<5)
#define	TE		(1<<3)
#define	RE		(1<<2)

#define	TXE		(1<<7)	/* Transmit data register empty */
#define	RXNE	(1<<5)	/* Receive data register full */

/*
 * USART1 Interrupt:
 * NUM=37, VEC = 0xD4
 * NVIC: Reg 2 Bitmask (1<<5)
 */
char inbuf, outbuf;

#define	NVIC_USART1_ISER 		0xE000E104  
#define	NVIC_USART1_ICPR 		0xE000E280  
#define	NVIC_USART1_IPR       	0xE000E400
#define NVIC_USART1_IRQ_BPOS  	(1<<5)



void	usart_irq_routine( void )
{
	if( USART1->sr & RXNE) 
		inbuf = (char) USART1->dr;
		
	if( ( USART1->cr1 & TXEIE) && ( USART1->sr & TXE) )
	{
		USART1->dr = (unsigned short) outbuf;
		USART1->cr1 &= ~TXEIE;	/* Clear interrupt on transmitter empty */		
	}
}

void	usart_init( void )
{
	*((void (**)(void) ) USART1_IRQVEC )  = usart_irq_routine;
	inbuf= 0;
	/*  Configure the enable and mask bits that control 
	* the NVIC IRQ channel mapped to USART1. */
    *((unsigned int *) NVIC_USART1_ISER) |= NVIC_USART1_IRQ_BPOS;	/* Enable this interrupt */ 
	/* Setup USART, receive with IRQ, polled transmit, 10 bit (1 startbit, 8 databits */ 
	USART1->brr = 0x2D9;	/* Baudrade = 115200 */
	USART1->cr3 = 0;
	USART1->cr2 = 0;	/* 1 stopbit */
	USART1->cr1 = EN | RXNEIE | TE | RE;
}

char	usart_tstchar ( void )
{
	char	c = inbuf;
	inbuf = 0;
	return c;
}
void	usart_outchar( char c )
{
	outbuf = c;
	USART1->cr1 |= TXEIE;	/* Interrupt on transmitter empty */

	
}

/* 
 * print character string to console 
 */
void	printlusart(char *s)
{
	while(*s)
		usart_outchar(*s++);
}

/*
 * usart receive/transmit interrupt test
 */
 
void main(void)
{	
	char	c;
	usart_init();
	while( 1 )
	{
		c = usart_tstchar();
		if ( c ){
            if( c == 'p')
                printlusart("USART program");
            else
                usart_outchar(c);
       }
	}
}

 
 