/*
 * 	fpexception.c
 *
 */
 #include	<float.h>
 #include	<stdio.h>

__attribute__((naked)) __attribute__((section (".start_section")) )
void startup ( void )
{
	__asm volatile (" LDR R0,=stack_top\n");
	__asm volatile (" MOV SP,R0\n");
	__asm volatile (" BL crt_init\n");
	__asm volatile (" BL main\n");
	__asm volatile (" BL crt_exit\n");
	__asm volatile (".L1: B .L1\n");
}


/*
 * 	FPU Global interrupt, nr 81
 * 81/32 = 2 -> NVIC_ISER2 = 0xE000E108
 * 81%32 = 17 -> NVIC_FPU_IRQ_BPOS (1<<17)
 */
#define	FPU_IRQVEC 	((void (**)(void) ) 0x2001C184 ) 

#define	NVIC_FPU_ISER 		0xE000E108  
#define NVIC_FPU_IRQ_BPOS  	(1<<17)

__attribute__((naked)) unsigned int getFPSCR( void )
{
	__asm volatile ("VMRS R0,FPSCR\n");
	__asm volatile ("BX LR\n");
}
__attribute__((naked)) void setFPSCR( unsigned int val )
{
	__asm volatile ("VMSR FPSCR,R0\n");
	__asm volatile ("BX LR\n");
}


void	fpu_irq_routine( void )
{
	char buffer[64];
	unsigned int fpscr;
	
	fpscr = getFPSCR();
	printf( "\nFloating Point Exception, FPSCR: 0x%8.8X", fpscr );
	if( fpscr & (1<<7 ))
		printf( "\nDenormalised");
	if( fpscr & (1<<4 ))
		printf( "\nInexact");
	if( fpscr & (1<<3 ))
		printf( "\nUnderflow");
	if( fpscr & (1<<2 ))
		printf( "\nOverflow");
	if( fpscr & (1<<1 ))
		printf( "\nZero divide");
	if( fpscr & (1<<0 ))
		printf( "\nIllegal operation");
		
	fpscr &= ~(0x9F);
	setFPSCR( fpscr );
}


typedef union float_long
{
    float 	f;
    long 	l;
} FLOAT_LONG;

float f;

void main(void)
{
	FLOAT_LONG fl1, fl2;
	
   *((void (**)(void) ) FPU_IRQVEC )  = fpu_irq_routine;
   *((unsigned int *) NVIC_FPU_ISER) |= NVIC_FPU_IRQ_BPOS; 

	 printf("\nCompare NaN");
	fl1.l = 0x3F800000;
	fl2.l = 0x7F800001;
	if(fl1.f == fl2.f) f = 1.0;
	
	 printf("\n\nAdd");
	fl1.f = FLT_MAX;
	fl2.f = FLT_MAX;
	f = fl1.f + fl2.f;

	 printf("\n\nDivide very large");
	fl1.f = FLT_MIN;
	fl2.f = FLT_MAX;
	f = fl1.f / fl2.f;

	 printf("\n\nDivide with 0");
	fl1.f = FLT_MAX;
	fl2.f = 0.0;
	f = fl1.f / fl2.f;
	
	 printf("\n\nDone");
}

