/*
 * Lab 3_4.c
 * Display till PE
 * Keypad till D8-15
 * Keypad-inmatning för att styra bollens rörelser
 */

#define	SIMULATOR
//#define	USBDM

__attribute__((naked)) 
__attribute__((section (".start_section")) )
void startup ( void )
{
__asm__ volatile(" LDR R0,=0x2001C000\n");		
__asm__ volatile(" MOV SP,R0\n");		
__asm__ volatile(" BL main\n");		
__asm__ volatile(" B .\n");		
}

#define		STK_CTRL 	((volatile unsigned int *)(0xE000E010))  
#define		STK_LOAD 	((volatile unsigned int *)(0xE000E014))  
#define		STK_VAL 	((volatile unsigned int *)(0xE000E018))  

/* FRÅN LAB 2.2 */
void delay_250ns( void )
{
	/* SystemCoreClock = 168000000 */
	*STK_CTRL = 0;
	*STK_LOAD = ( (168/4) -1 );
	*STK_VAL = 0;
	*STK_CTRL = 5;
	while( (*STK_CTRL & 0x10000 )== 0 );
	*STK_CTRL = 0;
}


void	delay_micro(unsigned int us)
{
	while( us > 0 )
	{
		delay_250ns();
		delay_250ns();
		delay_250ns();
		delay_250ns();
		us--;
	}
}

void	delay_milli(unsigned int ms)
{
	while( ms > 0 )
	{
#ifdef	SIMULATOR
		delay_micro( 1 );
#else
		delay_micro( 1000 );
#endif
		ms--;
	}
}



/* ---------------------------------------------------- */
/* Port D */
#define		GPIO_D_BASE		0x40020C00
#define		GPIO_D_MODER 	((volatile unsigned int *)	(GPIO_D_BASE))  
#define		GPIO_D_OTYPER 	((volatile unsigned short *)(GPIO_D_BASE+0x4))  
#define		GPIO_D_PUPDR 	((volatile unsigned int *)	(GPIO_D_BASE+0xC))  
#define		GPIO_D_IDRHIGH 	((volatile unsigned char *)(GPIO_D_BASE+0x11))  
#define		GPIO_D_ODRHIGH  	((volatile unsigned char *)	(GPIO_D_BASE+0x15))  
/* ----------------------------------------------------------------- */
    
__attribute__((naked)) 
void graphic_initalize(void)
{
	__asm volatile (" .HWORD	0xDFF0\n");	
	__asm volatile (" BX LR\n");		
}
 
__attribute__((naked)) 
void graphic_clearScreen(void)
{
    __asm volatile (" .HWORD	0xDFF1\n");		
	__asm volatile (" BX LR\n");		
}
 
__attribute__((naked))
void pixelset( int x, int y)
{
    __asm volatile (" .HWORD	0xDFF2\n");		
	__asm volatile (" BX LR\n");		
}
 
__attribute__((naked))
void pixelclear( int x, int y)
{
    __asm volatile (" .HWORD	0xDFF3\n");		
	__asm volatile (" BX LR\n");		
}
  
void pixel( int x, int y, int set )
{
    if( set )
    {
        pixelset( x, y);
    }else{
        pixelclear( x, y);	
    }
}

  
/* Objekt */
typedef struct tPoint
{
	unsigned char x;
	unsigned char y;
}POINT;

#define MAX_POINTS 25

typedef struct tGeometri
{
	int	numpoints;
	int	sizex;
	int	sizey;
	POINT	px[ MAX_POINTS ];
} GEOMETRY, *PGEOMETRY;
	

GEOMETRY spider_g =
{
	22,		// numpoints
	6,8,		// sizex,sizey
	{
		{0,2},		// px[]
		{0,3},
		{0,7},
		{1,1},
		{1,2},
		{1,4},
		{1,6},
		{2,0},
		{2,2},
		{2,3},
		{2,5},
		{3,0},
		{3,2},
		{3,3},
		{3,5},
		{4,1},
		{4,2},
		{4,4},
		{4,6},
		{5,2},
		{5,3},
		{3,7}
	}
};

GEOMETRY ball_g =
{
	12,		// numpoints
	4,4,		// sizex,sizey
	{
		{0,1},		// px[]
		{0,2},
		{1,0},
		{1,1},
		{1,2},
		{1,3},
		{2,0},
		{2,1},
		{2,2},
		{2,3},
		{3,1},
		{3,2}
	}
};


typedef struct tObj{
	PGEOMETRY geo;
	int	dirx,diry;
	unsigned int	posx,posy;
	void (* draw ) (struct tObj *);
	void (* clear ) (struct tObj *);
	void (* move ) (struct tObj *);
	void (* set_speed ) (struct tObj *, int, int);
} OBJECT, *POBJECT;





static void pixel_object(POBJECT o, int set)
{
	int	i;
	for( i = 0; i < o->geo->numpoints; i++ )
	{
		pixel ( o->geo->px[i].x + o->posx, o->geo->px[i].y + o->posy, set );
	}
}

void draw_object(POBJECT o)
{
	pixel_object(o, 1);
}


void clear_object(POBJECT o)
{
	pixel_object(o, 0);
}

void	set_object_speed(POBJECT o, int speedx, int speedy)
{ 
	o->dirx = speedx;
	o->diry = speedy;	
}


void	move_object(POBJECT o )
{
	/* move object in direction */
	int cx, cy;
	
	o->clear(o);	/* rubout current object */

	cx = o->posx + o->dirx;
	cy = o->posy + o->diry;

	/* should object bounce?  */
	if( cx+ o->geo->sizex > 128) 
	{
		o->dirx = - o->dirx;
		cx = 128 - o->geo->sizex;
	} 
	else if( cx<1 )
	{
		o->dirx = - o->dirx;
		cx = 1;
	}
	
	if( cy + o->geo->sizey > 64) 
	{
		o->diry = - o->diry;
		cy = 64 - o->geo->sizey;
	} 
	else if( cy<1 )
	{
		o->diry = - o->diry;
		cy = 1;
	}		
	/* save new position */
	o->posx =  cx;
	o->posy =  cy;
	o->draw(o);
}

int objects_overlap( POBJECT o1, POBJECT o2 )
{
	/* if object 1 overlaps object 2 return 1, else 0 */
	int o1xlow, o1xhigh, o1ylow, o1yhigh;
	int o2xlow, o2xhigh, o2ylow, o2yhigh;
	
	o1xlow = o1->posx;
	o1xhigh = o1xlow+o1->geo->sizex;
	o1ylow = o1->posy;
	o1yhigh = o1ylow+o1->geo->sizey;
	
	o2xlow = o2->posx;
	o2xhigh = o2xlow+o2->geo->sizex;
	o2ylow = o2->posy;
	o2yhigh = o2ylow+o2->geo->sizey;
	
	if( ( ( o2xhigh >= o1xlow ) &&  (o2xhigh <= o1xhigh ) ) || ( ( o2xlow >= o1xlow ) &&  (o2xlow <= o1xhigh ) ) )
	{
		if( ( ( o2yhigh >= o1ylow ) &&  (o2yhigh <= o1yhigh ) ) || ( ( o2ylow >= o1ylow ) &&  (o2ylow <= o1yhigh ) ) )
			return 1;
	}	
	return 0;
}


void init_app( void )
{
#ifdef USBDM	
	/* starta klockor port D och E */
	* ( (unsigned long *) 0x40023830) = 0x18;
	__asm( 	" BL =0x08000209\n");
#endif	
	/*	PORT D
 	b15-b12 used for output to rows
	b11-b8  used for input from columns
	b7-b0 	used for output to 7-seg display
	*/
	*GPIO_D_MODER = 		0x55005555;		
	*GPIO_D_PUPDR = 		0x00AA0000;		/* Input, pull down */
    *GPIO_D_OTYPER     = 0x00000000;	    /* outputs are push/pull */
}


/* Deklarera och instansiera ett objekt i form av en liten boll */
static OBJECT	ball =
{
	&ball_g,			// geometri för en boll
	2,1,				// initiala riktningskoordinater
	1,1,				// initial startposition
	draw_object,
	clear_object,
	move_object,	
	set_object_speed
};

/* Deklarera och instansiera ett objekt i form av en liten otrevlig varelse */
static OBJECT	spider =
{
	&spider_g,			// geometri för en otrevlig varelse
	0,0,				// initiala riktningskoordinater
	50,50,				// initial startposition
	draw_object,
	clear_object,
	move_object,	
	set_object_speed
};



void kbdActivate( unsigned int row )
{
	switch( row )
	{
		case	1: *GPIO_D_ODRHIGH = 0x10 ; 	break;
		case 	2: *GPIO_D_ODRHIGH = 0x20 ; 	break;
		case	3: *GPIO_D_ODRHIGH = 0x40 ; 	break;
		case	4: *GPIO_D_ODRHIGH = 0x80 ; 	break;
		case	0: *GPIO_D_ODRHIGH = 0x00; 	break;
	}
}

int kbdGetCol ( void )
{
	unsigned short c;
	c = *GPIO_D_IDRHIGH;
 	if ( c & 0x8 )		return 4;
	if ( c & 0x4 ) 	    return 3;
	if ( c & 0x2 ) 	    return 2;
	if ( c & 0x1 ) 	    return 1;
	return 0;
}




unsigned char keyb(void) 
{
	unsigned char key[]={1,2,3,0xA,4,5,6,0xB,7,8,9,0xC,0xE,0,0xF,0xD};
	int row, col;
	for (row=1; row <=4 ; row++ ) {
		kbdActivate( row );	
		if( (col = kbdGetCol () ) )
		{
			return key [4*(row-1)+(col-1) ];
		}
	}
	*GPIO_D_ODRHIGH = 0;
	return 0xFF;
}



/* ---------------------------------------------------- */




int main(int argc, char **argv)
{
	char	c;
	POBJECT victim = &ball;
	POBJECT creature = &spider;
	init_app();	
	graphic_initalize();
	graphic_clearScreen();
	victim->set_speed( victim, 4, 1);
	
	while( 1 )
	{
		victim->move( victim );
		creature->move( creature );
		c = keyb();
		switch( c )
		{
			case 6:	creature->set_speed( creature, 4, 0); break;	
			case 4:	creature->set_speed( creature, -4, 0); break;	
			case 5:	creature->set_speed( creature, 0, 0); break;	
			case 2:	creature->set_speed( creature, 0, -4); break;	
			case 8:	creature->set_speed( creature, 0, 4); break;
            default:
                creature->set_speed( creature, 0, 0); break;
		}
        if( objects_overlap( victim, creature ))
        {
            // Game over
            break;
        }
 		delay_milli(40);
	}
}

