/*
 * Lab 3_3.c
 * Automatiskt PONG-spel
 */


void startup(void) __attribute__((naked)) __attribute__((section (".start_section")) );

/* Built in DBG-routines, absolute addresses */
typedef void (* function1 )(void);  

#define hwInit  ((function1)    (0x08000201))   /* do hardware initialisation */
#define exitDBG ((function1)    (0x08000205))   /* reentry in debugger */

//#define GDB_DEBUG
#define SIMULATOR

static void Init( void )
{
#ifdef GDB_DEBUG
   hwInit();
#endif   
}

static void Exit( void )
{
    exitDBG();
}

void startup ( void )
{
asm volatile(
	" NOP\n"					/* executed by first 'trace'-command */
	" LDR SP,=0x2001C000\n"		/* set stack */
	" BL Init\n"				/* do hardware init */
	" BL main\n"				/* call main */
	" B  Exit\n"				/* exit to debugger */
	".L1: B .L1\n"				/* never return */
	) ;
}



#define		SysTickCtrl 	((volatile unsigned int *)(0xE000E010))  
#define		SysTickLoad 	((volatile unsigned int *)(0xE000E014))  
#define		SysTickVal 		((volatile unsigned int *)(0xE000E018))  

/* FRÅN LAB 2.2 */
void delay_250ns( void )
{
#ifdef	SIMULATOR
	return;
#endif
	/* SystemCoreClock = 168000000 */
	*SysTickCtrl = 0;
	*SysTickLoad = ( (168/4) -1 );
	*SysTickVal = 0;
	*SysTickCtrl = 5;
	while( (*SysTickCtrl & 0x10000 )== 0 );
	*SysTickCtrl = 0;
}


void	delay_micro(unsigned int us)
{
#ifdef	SIMULATOR
	return;
#endif
	while( us > 0 )
	{
		delay_250ns();
		delay_250ns();
		delay_250ns();
		delay_250ns();
		us--;
	}
}

void	delay_milli(unsigned int ms)
{
#ifdef	SIMULATOR
	return;
#endif
	while( ms > 0 )
	{
		delay_micro( 1 );
		delay_micro( 1000 );
		ms--;
	}
}



void delay_500ns( void )
{
#ifdef	SIMULATOR
	return;
#endif
	delay_250ns();
	delay_250ns();
}

/* ----------------------------------------------------------------- */



#define		PORT_DISPLAY_BASE		0x40021000		/* MD407 port E */


#define	B_E		0x40
#define	B_RST	0x20
#define	B_CS2	0x10
#define	B_CS1	8
#define	B_SELECT	4			
#define	B_RW	2
#define	B_DI	1
#define	B_RS	1

#define LCD_ON				0x3F
#define LCD_OFF				0x3E
#define LCD_SET_ADD			0x40
#define LCD_SET_PAGE		0xB8
#define LCD_DISP_START		0xC0
#define LCD_BUSY			0x80

#define		portModer 		((volatile unsigned int *)	(PORT_DISPLAY_BASE))  
#define		portOtyper 		((volatile unsigned short *)(PORT_DISPLAY_BASE+0x4))  
#define		portOspeedr 	((volatile unsigned int *)	(PORT_DISPLAY_BASE+0x8))  
#define		portPupdr 		((volatile unsigned int *)	(PORT_DISPLAY_BASE+0xC))  
#define		portIdr 		((volatile unsigned short *)(PORT_DISPLAY_BASE+0x10))  
#define		portIdrHigh		((volatile unsigned char *)(PORT_DISPLAY_BASE+0x11))  
#define		portOdrLow 		((volatile unsigned char *)(PORT_DISPLAY_BASE+0x14))  
#define		portOdrHigh 	((volatile unsigned char *)(PORT_DISPLAY_BASE+0x14+1))  


void ascii_ctrl_bit_set( unsigned char x  )
{
	unsigned char c;
	c = *portOdrLow;
	c |= ( B_SELECT | x );
	*portOdrLow = c;
}
void	ascii_ctrl_bit_clear( unsigned char x )
{	
	unsigned char c;
	c = *portOdrLow;
	c &= ( B_SELECT | ~x );
	*portOdrLow = c;
}



void	ascii_write_controller( unsigned char c )
{
	ascii_ctrl_bit_set( B_E );	
	*portOdrHigh = c;
	ascii_ctrl_bit_clear( B_E );
	delay_250ns();
}

void	ascii_write_cmd( unsigned char c)
{
	ascii_ctrl_bit_clear( B_RS | B_RW );
	ascii_write_controller( c );
}


void	ascii_write_data( unsigned char c)
{
	ascii_ctrl_bit_clear( B_RW );
	ascii_ctrl_bit_set( B_RS );
	ascii_write_controller( c );
}


unsigned char ascii_read_controller( void )
{
	unsigned char c;
	ascii_ctrl_bit_set( B_E );	
	delay_250ns();	/* max 360 ns */
	delay_250ns();	
	c = *portIdrHigh;
	ascii_ctrl_bit_clear( B_E );
	return c;
}


unsigned char ascii_read_status()
{
	unsigned char c;
	*portModer	= 0x00005555;	/* b15-8 are inputs, 7-0 are outputs */		
	ascii_ctrl_bit_set( B_RW );
	ascii_ctrl_bit_clear( B_RS );	
	c = ascii_read_controller(  );
	*portModer	 	= 0x55555555;	/* all bits outputs */
	return c;
}

void ascii_wait_ready(void)
{
	while( ( ascii_read_status() & 0x80)== 0x80 );
	delay_micro( 8 );		/* se not sid 16 */

}

void	ascii_write_char( unsigned char c)
{
	ascii_wait_ready();
	ascii_write_data( c);
	delay_micro( 39 );
}

void ascii_init()
{
	ascii_wait_ready();
	ascii_write_cmd( 0x38 );	/* Function set */
	delay_micro( 39 );
	ascii_wait_ready();
	ascii_write_cmd( 0x0C );	/* Display on */
	delay_micro( 39 );
	ascii_wait_ready();
	ascii_write_cmd( 1 );		/* Clear display */
	delay_milli( 2 );
	ascii_wait_ready();
	ascii_write_cmd( 6 );		/* Entry mode set */
	delay_micro( 39 );
}




void ascii_gotoxy( unsigned char x, unsigned char y) {
   unsigned char address;

   if(y!=1)
     address=0x40 | (x-1);
   else
     address=x-1;
   ascii_write_cmd( 0x80 | address);
}

void	ascii_write_string( unsigned char *cp)
{
	ascii_gotoxy(1,1);
	while( *cp )
		ascii_write_char( *cp++ );
}


/* Graphic display */
static void	graphic_ctrl_bit_set( unsigned char x )
{	
	*portOdrLow|= ( ~B_SELECT & x );
}
static void	graphic_ctrl_bit_clear( unsigned char x )
{
	*portOdrLow &= ( ~B_SELECT & ~x );
}

static void select_controller(unsigned char controller)
{
switch(controller){
	case 0:				graphic_ctrl_bit_clear(B_CS1|B_CS2);  break;
	case B_CS1 : 		graphic_ctrl_bit_set(B_CS1); graphic_ctrl_bit_clear(B_CS2); break;
	case B_CS2 : 		graphic_ctrl_bit_set(B_CS2); graphic_ctrl_bit_clear(B_CS1);  break;
	case B_CS1|B_CS2 : 	graphic_ctrl_bit_set(B_CS1|B_CS2);  break;
	}
}


static void graphic_wait_ready(void)
{
	unsigned char c;
	
	graphic_ctrl_bit_clear( B_E ); 
	*portModer	= 0x00005555;	/* b15-8 are inputs, 7-0 are outputs */	
	graphic_ctrl_bit_clear( B_DI ); 
	graphic_ctrl_bit_set( B_RW ); 
	delay_500ns();
	
	while( 1 )
	{
		graphic_ctrl_bit_set( B_E ); 
		delay_500ns();
		c = *portIdrHigh & 0x80;	
		graphic_ctrl_bit_clear( B_E ); 
		if( c == 0 )break;
		delay_500ns();
	}
	*portModer	 = 0x55555555;	/* all bits outputs */
//	graphic_ctrl_bit_set( B_E );
}

static unsigned char  display_read(unsigned char controller)
{
	unsigned char c;
	*portModer	= 0x00005555;	/* b15-8 are inputs, 7-0 are outputs */	
	select_controller( controller );

	graphic_ctrl_bit_clear( B_E ); 
	graphic_ctrl_bit_set( B_DI | B_RW ); 
	delay_500ns();
	graphic_ctrl_bit_set( B_E ); 
	delay_500ns();
	c = *portIdrHigh;	

	graphic_ctrl_bit_clear( B_E );
	*portModer	 = 0x55555555;	/* all bits outputs */


	if( controller & B_CS1 )
	{
		select_controller( B_CS1);
		graphic_wait_ready();
	}
	if( controller & B_CS2 )
	{
		select_controller( B_CS2);
		graphic_wait_ready();
	}
	return c;
}



static void graphic_write(unsigned char val, unsigned char controller)
{
	*portOdrHigh = val;
	select_controller( controller );
	delay_500ns();
	graphic_ctrl_bit_set( B_E ); 
	delay_500ns();
	graphic_ctrl_bit_clear( B_E ); 
	
	if( controller & B_CS1 )
	{
		select_controller( B_CS1);
		graphic_wait_ready();
	}
	if( controller & B_CS2 )
	{
		select_controller( B_CS2);
		graphic_wait_ready();
	}
	
	*portOdrHigh = 0;
	graphic_ctrl_bit_set( B_E ); 
	select_controller( 0 );
}




static unsigned char  graphic_read(unsigned char controller)
{
	display_read(controller);
	return display_read(controller);
}



static void graphic_writeCommand(unsigned char commandToWrite, unsigned char controller)
{
	graphic_ctrl_bit_clear( B_E ); 
	graphic_ctrl_bit_clear( B_DI | B_RW ); 
	graphic_write(commandToWrite, controller);
}


static void graphic_writeData(unsigned char data, unsigned char controller)
{
	graphic_ctrl_bit_clear( B_E ); 
	graphic_ctrl_bit_set( B_DI ); 
	graphic_ctrl_bit_clear( B_RW ); 
	graphic_write(data, controller);
}



void graphic_clearScreen(void)
{
#ifdef	SIMULATOR
		return;
#else
	
	
	unsigned char i, j;
	for(j = 0; j < 8; j++)
	{
		graphic_writeCommand(LCD_SET_PAGE | j, B_CS1|B_CS2 );
		graphic_writeCommand(LCD_SET_ADD | 0, B_CS1|B_CS2 );
		for(i = 0; i <= 63; i++){
			graphic_writeData(0,  B_CS1|B_CS2);
		}
  }
#endif
}



void pixel( int x, int y, int set )
{
	unsigned char mask,c, controller;
	int	index;
	
	if( (x > 128 ) || (y > 64) )
	return;
	
	index = (y-1)/8;
	switch( (y-1)%8 )
	{
		case 0: mask = 1; break;
		case 1: mask = 2; break;
		case 2: mask = 4; break;
		case 3: mask = 8; break;
		case 4: mask = 0x10; break;
		case 5: mask = 0x20; break;
		case 6: mask = 0x40; break;
		case 7: mask = 0x80; break;
	}
	if( set == 0)
		mask = ~mask;
		

	if(x > 64){
		controller = B_CS2;
		x = x - 65;
	}else{
		controller = B_CS1;
		x = x-1;
	}
		
	graphic_writeCommand(LCD_SET_ADD | x, controller );
	graphic_writeCommand(LCD_SET_PAGE | index, controller );
	c = graphic_read( controller );
	graphic_writeCommand(LCD_SET_ADD | x, controller );
	if( set )
		mask = mask | c;
	else
		mask = mask & c;
	graphic_writeData( mask, controller);
}


void graphic_initalize(void)
{
	graphic_ctrl_bit_set( B_E ); 
	delay_micro(10);

	graphic_ctrl_bit_clear(B_CS1|B_CS2|B_RST|B_E);  
	delay_milli( 30 );
	graphic_ctrl_bit_set(B_RST); 	
	delay_milli( 100 );
	graphic_writeCommand(LCD_OFF, B_CS1|B_CS2);
	graphic_writeCommand(LCD_ON, B_CS1|B_CS2);
	graphic_writeCommand(LCD_DISP_START, B_CS1|B_CS2);
	graphic_writeCommand(LCD_SET_ADD, B_CS1|B_CS2);
	graphic_writeCommand(LCD_SET_PAGE, B_CS1|B_CS2);
	select_controller(0);
}


void	init_app(void)
{
	/* PORT E */
	*portModer	 	= 0x55555555;	/* all bits outputs */
	*portOtyper 	= 0x00000000;	/* outputs are push/pull */	
	*portOspeedr 	= 0x55555555;	/* medium speed */
	*portPupdr 		= 0x55550000;	/* inputs are pull up */
}


/* Objekt */
typedef struct tPoint
{
	unsigned char x;
	unsigned char y;
}POINT;

#define MAX_POINTS 20

typedef struct tGeometri
{
	int	numpoints;
	int	sizex;
	int	sizey;
	POINT	px[ MAX_POINTS ];
} GEOMETRY, *PGEOMETRY;
	

GEOMETRY ball_g =
{
	12,		// numpoints
	4,4,		// sizex,sizey
	{
		{0,1},		// px[]
		{0,2},
		{1,0},
		{1,1},
		{1,2},
		{1,3},
		{2,0},
		{2,1},
		{2,2},
		{2,3},
		{3,1},
		{3,2}
	}
};

typedef struct tObj{
	PGEOMETRY geo;
	int	dirx,diry;
	unsigned int	posx,posy;
	void (* draw ) (struct tObj *);
	void (* clear ) (struct tObj *);
	void (* move ) (struct tObj *);
	void (* set_speed ) (struct tObj *, int, int);
} OBJECT, *POBJECT;





static void pixel_object(POBJECT o, int set)
{
	int	i;
	for( i = 0; i < o->geo->numpoints; i++ )
	{
		pixel ( o->geo->px[i].x + o->posx, o->geo->px[i].y + o->posy, set );
	}
	
}

void draw_object(POBJECT o)
{
	pixel_object(o, 1);
}


void clear_object(POBJECT o)
{
	pixel_object(o, 0);
}

void	set_object_speed(POBJECT o, int speedx, int speedy)
{ 
	o->dirx = speedx;
	o->diry = speedy;	
}


void	move_object(POBJECT o )
{
	/* move object in direction */
	int cx, cy;
	
	o->clear(o);	/* rubout current object */

	cx = o->posx + o->dirx;
	cy = o->posy + o->diry;

	/* should object bounce?  */
	if( cx+ o->geo->sizex > 128) 
	{
		o->dirx = - o->dirx;
		cx = 128 - o->geo->sizex;
	} 
	else if( cx<1 )
	{
		o->dirx = - o->dirx;
		cx = 1;
	}
	
	if( cy + o->geo->sizey > 64) 
	{
		o->diry = - o->diry;
		cy = 64 - o->geo->sizey;
	} 
	else if( cy<1 )
	{
		o->diry = - o->diry;
		cy = 1;
	}		
	/* save new position */
	o->posx =  cx;
	o->posy =  cy;
	o->draw(o);
}



/* Deklarera och instansiera ett objekt i form av en liten boll */
static OBJECT	ball =
{
	&ball_g,			// geometri för en boll
	0,0,				// initiala riktningskoordinater
	1,1,				// initial startposition
	draw_object,
	clear_object,
	move_object,	
	set_object_speed
};


#if 0
// Detta är obligatoriskt,, uppgift 3.3
int main(int argc, char **argv)
{
	POBJECT p = &ball;
	init_app();	
	graphic_initalize();
	graphic_clearScreen();
	p->set_speed( p, 8, 1);
	
	while( 1 )
	{
		p->move( p );
		delay_milli(40);	/* 25 frames/sec */
	}
}
#else
// Frivillig uppgift

#define	USART_DATA_REG	( (unsigned short *) 0x40011004 )
#define	USART_SR_REG	( (unsigned short *) 0x40011000 )
void	_outchar( char c )
{
	/* write character to usart1 */
	*USART_DATA_REG = (unsigned short) c;
 	while (( *USART_SR_REG & 0x80)==0);
	if( c == '\n')
			_outchar('\r');
}


char	_tstchar( void )
{
	if( (*USART_SR_REG & 0x20)==0)
		return 0;
	return (char) *USART_DATA_REG;
}




int main(int argc, char **argv)
{
	char	c;
	POBJECT p = &ball;
	init_app();	
	ascii_init();
	ascii_gotoxy(1,1);
	graphic_initalize();
	graphic_clearScreen();
	ascii_write_string( "Use Arrows to change direction");

	
	while( 1 )
	{
		p->move( p );
		delay_milli(40);	/* 25 frames/sec */
		c = _tstchar();
		switch( c )
		{
			case '6':	p->set_speed( p, 2, 0); break;	
			case '4':	p->set_speed( p, -2, 0); break;	
			case '8':	p->set_speed( p, 0, -2); break;	
			case '2':	p->set_speed( p, 0, 2); break;	

		}
	}
}
#endif
