/*
 * 	libMD407
 *  keypad_driver.c
 *  Keypad connected to PD8-PD15
 *  Driver for KEYPAD
 */
#include	"libMD407.h"

static int keypad_init( int initval );
static void keypad_deinit( int deinitval);
static int keypad_read(char *ptr, int len);

DEV_DRIVER_DESC KeyPad = 
{
  {"Keypad"},
  keypad_init,
  keypad_deinit,
  0,
  0,
  0,
  0,
  0,
  0,
  keypad_read
};

/* Define functions here */



/* ---------------------------------------------------- */
/* Port D */
#define		PORT_D_BASE		0x40020C00
#define		GPIO_D_MODER 	((volatile unsigned int *)	(PORT_D_BASE))  
#define		GPIO_D_OTYPER 	((volatile unsigned short *)(PORT_D_BASE+0x4))  
#define		GPIO_D_OSPEEDR 	((volatile unsigned int *)	(PORT_D_BASE+0x8))  
#define		GPIO_D_PUPDR 	((volatile unsigned int *)	(PORT_D_BASE+0xC))  
#define		GPIO_D_IDRLOW 	((volatile unsigned char *)(PORT_D_BASE+0x10))  
#define		GPIO_D_IDRHIGH 	((volatile unsigned char *)(PORT_D_BASE+0x11))  
#define		GPIO_D_ODRLOW 	((volatile unsigned char *)	(PORT_D_BASE+0x14))  
#define		GPIO_D_ODRHIGH 	((volatile unsigned char *)	(PORT_D_BASE+0x15))  



static int keypad_init( int initval )
{
	/*	PORT D
 	b15-b12 used for output to rows
	b11-b8  used for input from columns
	b7-b0 	used for output to 7-seg display
	*/
	*GPIO_D_MODER = 0x55005555;		
	*GPIO_D_PUPDR = 0x00AA0000;		/* Input, pull down */
    *GPIO_D_OTYPER= 0x00000000;	    /* outputs are push/pull */

	*GPIO_D_ODRLOW = 0;
	*GPIO_D_ODRHIGH = 0;
    return 1; /* Ok init */
}

static void keypad_deinit( int initval )
{
    
}


static void kbdActivate( unsigned int row )
{
	switch( row )
	{
		case	1: *GPIO_D_ODRHIGH = 0x10 ; 	break;
		case 	2: *GPIO_D_ODRHIGH = 0x20 ; 	break;
		case	3: *GPIO_D_ODRHIGH = 0x40 ; 	break;
		case	4: *GPIO_D_ODRHIGH = 0x80 ; 	break;
		case	0: *GPIO_D_ODRHIGH = 0x00; 	break;
	}
}

static int kbdGetCol ( void )
{
	unsigned short c;
	c = *GPIO_D_IDRHIGH;
 	if ( c & 0x8 )		return 4;
	if ( c & 0x4 ) 	    return 3;
	if ( c & 0x2 ) 	    return 2;
	if ( c & 0x1 ) 	    return 1;
	return 0;
}

static unsigned char keyb(void) 
{
	unsigned char key[]={1,2,3,0xA,4,5,6,0xB,7,8,9,0xC,0xE,0,0xF,0xD};
	int row, col;
	for (row=1; row <=4 ; row++ ) {
		kbdActivate( row );	
		if( (col = kbdGetCol () ) )
		{
			return key [4*(row-1)+(col-1) ];
		}
	}
	*GPIO_D_ODRHIGH = 0;
	return 0xFF;
}

static int keypad_read(char *ptr, int len)
{
    char c = keyb();
    if( c != 0xFF )
    {
        *ptr=c;
        return 1;
    }
    return 0;
 }
