/*
 * 	libMD407
 *  asciidisplay_driver.c
 *  Display connected to PE
 *  Driver for ASCIIDISPLAY
 */
#include	"libMD407.h"

static int asciidisplay_init( int initval );
static void asciidisplay_deinit( int deinitval);
static int asciidisplay_write(char *ptr, int len);

DEV_DRIVER_DESC AsciiDisplay = 
{
  {"AsciiDisplay"},
  asciidisplay_init,
  asciidisplay_deinit,
  0,
  0,
  0,
  0,
  0,
  asciidisplay_write,
  0
};

/* Define functions here */

/* Från lab 2.2 */
#define		STK_CTRL 	((volatile unsigned int *)(0xE000E010))  
#define		STK_LOAD 	((volatile unsigned int *)(0xE000E014))  
#define		STK_VAL 	((volatile unsigned int *)(0xE000E018))  

void delay_250ns( void )
{
#ifdef	SIMULATOR
	return;
#endif
	/* SystemCoreClock = 168000000 */
	*STK_CTRL = 0;
	*STK_LOAD = ( (168/4) -1 );
	*STK_VAL = 0;
	*STK_CTRL = 5;
	while( (*STK_CTRL & 0x10000 )== 0 );
	*STK_CTRL = 0;
}


void	delay_micro(unsigned int us)
{
	while( us > 0 )
	{
		delay_250ns();
		delay_250ns();
		delay_250ns();
		delay_250ns();
		us--;
	}
}

void	delay_milli(unsigned int ms)
{
#ifdef	SIMULATOR
	ms = ms / 1000;
	ms++;
#endif
	while( ms > 0 )
	{
		delay_micro( 1 );
		delay_micro( 1000 );
		ms--;
	}
}



/* ----------------------------------------------------------------- */


#define	B_E			0x40
#define	B_SELECT    4
#define	B_RW		2
#define	B_RS		1

#define		GPIO_E_BASE		0x40021000		/* MD407 port E */

#define		GPIO_E_MODER 	((volatile unsigned int *)	(GPIO_E_BASE))  
#define		GPIO_E_OTYPER 	((volatile unsigned short *)(GPIO_E_BASE+0x4))  
#define		GPIO_E_OSPEEDR 	((volatile unsigned int *)	(GPIO_E_BASE+0x8))  
#define		GPIO_E_PUPDR 	((volatile unsigned int *)	(GPIO_E_BASE+0xC))  

/* Byte address definitions */
#define		GPIO_E_IDRHIGH 	((volatile unsigned char *)	(GPIO_E_BASE+0x10+1))  
#define		GPIO_E_ODRLOW 	((volatile unsigned char *)	(GPIO_E_BASE+0x14))  
#define		GPIO_E_ODRHIGH 	((volatile unsigned char *)	(GPIO_E_BASE+0x14+1))  

#define ascii_ctrl_bit_set(x)  \
    *GPIO_E_ODRLOW = \
    (B_SELECT | (x) | *GPIO_E_ODRLOW)


#define ascii_ctrl_bit_clear(x) \
	*GPIO_E_ODRLOW = \
	(B_SELECT | ( *GPIO_E_ODRLOW & ~(x) ))


void	ascii_write_controller( unsigned char c )
{
	ascii_ctrl_bit_set( B_E );	
	*GPIO_E_ODRHIGH = c;
	ascii_ctrl_bit_clear( B_E );
	delay_250ns();
}

void	ascii_write_cmd( unsigned char c)
{
	ascii_ctrl_bit_clear( B_RS | B_RW );
	ascii_write_controller( c );
}

void	ascii_write_data( unsigned char c)
{
	ascii_ctrl_bit_clear( B_RW );
	ascii_ctrl_bit_set( B_RS );
	ascii_write_controller( c );
}

unsigned char ascii_read_controller( void )
{
	unsigned char c;
	ascii_ctrl_bit_set( B_E );	
	delay_250ns();	/* max 360 ns */
	delay_250ns();	
	c = *GPIO_E_IDRHIGH;
	ascii_ctrl_bit_clear( B_E );
	return c;
}

unsigned char ascii_read_status()
{
	unsigned char c;
	*GPIO_E_MODER	= 0x00005555;	/* b15-8 are inputs, 7-0 are outputs */		
	ascii_ctrl_bit_set( B_RW );
	ascii_ctrl_bit_clear( B_RS );	
	c = ascii_read_controller(  );
	*GPIO_E_MODER	 	= 0x55555555;	/* all bits outputs */
	return c;
}

void ascii_wait_ready(void)
{
	while( ( ascii_read_status() & 0x80)== 0x80 );
	delay_micro( 8 );		/* se not sid 16 */

}

void	ascii_write_char( unsigned char c)
{
	ascii_wait_ready();
	ascii_write_data( c);
	delay_micro( 39 );
}




void ascii_gotoxy( unsigned char x, unsigned char y) {
   unsigned char address;

   if(y!=1)
     address=0x40 | (x-1);
   else
     address=x-1;
   ascii_write_cmd( 0x80 | address);
}

static int asciidisplay_write(char *ptr, int len)
{   /* TODO, position och radhantering... */
    int rval = len;
 	while (len--);
      ascii_write_char(*ptr++);
}



static int asciidisplay_init( int initval )
{
#ifdef USBDM	
	/* starta klockor port D och E */
	* ( (unsigned long *) 0x40023830) = 0x18;
	/* initiera PLL */
	__asm volatile( " LDR R0,=0x08000209\n BLX R0 \n");
#endif	
	/* PORT E */
	*GPIO_E_MODER	 	= 0x55555555;	/* all bits outputs */
	*GPIO_E_OTYPER 	= 0x00000000;	/* outputs are push/pull */	
	*GPIO_E_OSPEEDR 	= 0x55555555;	/* medium speed */
    
	ascii_wait_ready();
	ascii_write_cmd( 0x38 );	/* Function set */
	delay_micro( 39 );
	ascii_wait_ready();
	ascii_write_cmd( 0x0C );	/* Display on */
	delay_micro( 39 );
	ascii_wait_ready();
	ascii_write_cmd( 1 );		/* Clear display */
	delay_milli( 2 );
	ascii_wait_ready();
	ascii_write_cmd( 6 );		/* Entry mode set */
	delay_micro( 39 );
    return 1;
}

static void asciidisplay_deinit( int initval )
{
}