/*
 * kbd_alt_ctrl.c 
 * Keypad med alternativa tangentkoder
 * Keypad ansluts till PD8-15
 * 7-segmentsdisplay ansluts till PD0-7
 */

__attribute__((naked)) 
__attribute__((section (".start_section")) )
void startup ( void )
{
__asm__ volatile(" LDR R0,=0x2001C000\n");		
__asm__ volatile(" MOV SP,R0\n");		
__asm__ volatile(" BL main\n");		
__asm__ volatile(" B .\n");		
}

/* ---------------------------------------------------- */
/* Port D */
#define		PORT_D_BASE		0x40020C00
#define		GPIO_D_MODER 	((volatile unsigned int *)	(PORT_D_BASE))  
#define		GPIO_D_OTYPER 	((volatile unsigned short *)(PORT_D_BASE+0x4))  
#define		GPIO_D_OSPEEDR 	((volatile unsigned int *)	(PORT_D_BASE+0x8))  
#define		GPIO_D_PUPDR 	((volatile unsigned int *)	(PORT_D_BASE+0xC))  
#define		GPIO_D_IDRLOW 	((volatile unsigned char *)(PORT_D_BASE+0x10))  
#define		GPIO_D_IDRHIGH 	((volatile unsigned char *)(PORT_D_BASE+0x11))  
#define		GPIO_D_ODRLOW 	((volatile unsigned char *)	(PORT_D_BASE+0x14))  
#define		GPIO_D_ODRHIGH 	((volatile unsigned char *)	(PORT_D_BASE+0x15))  

/* Bitmasker för tangenter */
#define NUMERIC 0x7772
#define Ctrl	0x80
#define	Alt 	0x8000

void init_app( void )
{
	/*	PORT D
 	b15-b12 used for output to rows
	b11-b8  used for input from columns
	b7-b0 	used for output to 7-seg display
	*/
	*GPIO_D_MODER = 0x55005555;		
	*GPIO_D_PUPDR = 0x00AA0000;		/* Input, pull down */
    *GPIO_D_OTYPER= 0x00000000;	    /* outputs are push/pull */
    *GPIO_D_OSPEEDR= 0xAA000000;	   /* outputs speed */

	*GPIO_D_ODRLOW = 0;
	*GPIO_D_ODRHIGH = 0;
}

unsigned short keyb_alt_ctrl(void) 
{
	/* Kommentar: 
	 * Här är det viktigt att uppdateringsfrekvensen 
	 * i laborationssystemet är tillräcklig (se OSPEEDR i init_app)
	 * det behövs inte i simulatorn.
	 */
	int row;
	unsigned short pad;
	*GPIO_D_ODRHIGH = 0x80;
	pad = (unsigned short) (*GPIO_D_IDRHIGH & 0xF);
	*GPIO_D_ODRHIGH = 0x40;
	pad |= (unsigned short)  ((*GPIO_D_IDRHIGH <<4) & 0x00F0);
	*GPIO_D_ODRHIGH = 0x20;
	pad |= (unsigned short)  ((*GPIO_D_IDRHIGH <<8) & 0x0F00);
	*GPIO_D_ODRHIGH = 0x10;
	pad |= (unsigned short)  ((*GPIO_D_IDRHIGH <<12) & 0xF000);
	*GPIO_D_ODRHIGH = 0;
	return pad;
}

unsigned char is_numeric( unsigned short s )
{
	if( s != 0)
	{		
		switch( s & NUMERIC )
		{
			case 0x1000: return 1;
			case 0x2000: return 2;
			case 0x4000: return 3;
			case 0x0100: return 4;
			case 0x0200: return 5;
			case 0x0400: return 6;
			case 0x0010: return 7;
			case 0x0020: return 8;
			case 0x0040: return 9;
			case 0x0002: return 0;
		}
	}
	return 0xFF;
}

void	out7seg( unsigned char c )
{
static unsigned char segCode[]={ 
	0x3F,0x06,0x5B,0x4F,0x66,0x6D,0x7D,0x07,
	0x7F,0x67,0x77,0x7C,0x39,0x5E,0x79,0x71 };
	
	if( c > 15){
		*GPIO_D_ODRLOW = 0;
		return;
	}
	*GPIO_D_ODRLOW = segCode[c];
}

/* ---------------------------------------------------- */
void main(void)
{
	unsigned short s; 
	unsigned char c; 
	int dp;
    init_app();
	
	while( 1 )
	{
       s = keyb_alt_ctrl();
	   for( int i = 0; i < 100; i++ );	/* HW fördröj */

	   c = is_numeric( s ); 
	   if( c != 0xFF )
			out7seg( c );
	}
}

