/*
 * asciidisplay.c
 * LCD-modul DATA   PE8-15
 * LCD-modul CTRL   PE0-7
 */

#define SIMULATOR
//#define USBDM

__attribute__((naked)) 
__attribute__((section (".start_section")) )
void startup ( void )
{
__asm__ volatile(" LDR R0,=0x2001C000\n");		
__asm__ volatile(" MOV SP,R0\n");		
__asm__ volatile(" BL main\n");		
__asm__ volatile(" B .\n");		
}

#define		GPIO_E_BASE		0x40021000		/* MD407 port E */
#define		GPIO_E_MODER 	((volatile unsigned int *)	(GPIO_E_BASE))  
#define		GPIO_E_OTYPER 	((volatile unsigned short *)(GPIO_E_BASE+0x4))  
#define		GPIO_E_OSPEEDR 	((volatile unsigned int *)	(GPIO_E_BASE+0x8))  
#define		GPIO_E_PUPDR 	((volatile unsigned int *)	(GPIO_E_BASE+0xC))  
/* Byte address definitions */
#define		GPIO_E_IDRHIGH 	((volatile unsigned char *)	(GPIO_E_BASE+0x10+1))  
#define		GPIO_E_ODRLOW 	((volatile unsigned char *)	(GPIO_E_BASE+0x14))  
#define		GPIO_E_ODRHIGH 	((volatile unsigned char *)	(GPIO_E_BASE+0x14+1))  

/* SysTick */
#define		STK_CTRL 	((volatile unsigned int *)(0xE000E010))  
#define		STK_LOAD 	((volatile unsigned int *)(0xE000E014))  
#define		STK_VAL 	((volatile unsigned int *)(0xE000E018))  

/* Bit definitions for Display interface */
#define	B_E			0x40
#define	B_SELECT    4
#define	B_RW		2
#define	B_RS		1

void	init_app(void)
{
	/* PORT E */
	*GPIO_E_MODER	= 0x55555555;	/* all bits outputs */
	*GPIO_E_OTYPER 	= 0x00000000;	/* outputs are push/pull */	
	*GPIO_E_OSPEEDR = 0x55555555;	/* medium speed */
	*GPIO_E_PUPDR 	= 0x55550000;	/* inputs are pull up */
}

void delay_250ns( void )
{
#ifdef	SIMULATOR
	return;
#endif
	/* SystemCoreClock = 168000000 */
	*STK_CTRL = 0;
	*STK_LOAD = ( (168/4) -1 );
	*STK_VAL = 0;
	*STK_CTRL = 5;
	while( (*STK_CTRL & 0x10000 )== 0 );
	*STK_CTRL = 0;
}


void	delay_micro(unsigned int us)
{
	while( us > 0 )
	{
		delay_250ns();
		delay_250ns();
		delay_250ns();
		delay_250ns();
		us--;
	}
}

void	delay_milli(unsigned int ms)
{
#ifdef	SIMULATOR
	ms = ms / 1000;
	ms++;
#endif
	while( ms > 0 )
	{
		delay_micro( 1 );
		delay_micro( 1000 );
		ms--;
	}
}



/* ----------------------------------------------------------------- */




#define ascii_ctrl_bit_set(x)  \
    *GPIO_E_ODRLOW = \
    (B_SELECT | (x) | *GPIO_E_ODRLOW)


#define ascii_ctrl_bit_clear(x) \
	*GPIO_E_ODRLOW = \
	(B_SELECT | ( *GPIO_E_ODRLOW & ~(x) ))

/*
void ascii_ctrl_bit_set( unsigned char x  )
{
	unsigned char c;
	c = *GPIO_E_ODRLOW;
	c |= ( B_SELECT | x );
	*GPIO_E_ODRLOW = c;
}
void	ascii_ctrl_bit_clear( unsigned char x )
{	
	unsigned char c;
	c = *GPIO_E_ODRLOW;
	c &= ( B_SELECT | ~x );
	*GPIO_E_ODRLOW = c;
}
*/

void	ascii_write_controller( unsigned char c )
{
	ascii_ctrl_bit_set( B_E );	
	*GPIO_E_ODRHIGH = c;
	ascii_ctrl_bit_clear( B_E );
	delay_250ns();
}

void	ascii_write_cmd( unsigned char c)
{
	ascii_ctrl_bit_clear( B_RS | B_RW );
	ascii_write_controller( c );
}

void	ascii_write_data( unsigned char c)
{
	ascii_ctrl_bit_clear( B_RW );
	ascii_ctrl_bit_set( B_RS );
	ascii_write_controller( c );
}

unsigned char ascii_read_controller( void )
{
	unsigned char c;
	ascii_ctrl_bit_set( B_E );	
	delay_250ns();	/* max 360 ns */
	delay_250ns();	
	c = *GPIO_E_IDRHIGH;
	ascii_ctrl_bit_clear( B_E );
	return c;
}

unsigned char ascii_read_status()
{
	unsigned char c;
	*GPIO_E_MODER	= 0x00005555;	/* b15-8 are inputs, 7-0 are outputs */		
	ascii_ctrl_bit_set( B_RW );
	ascii_ctrl_bit_clear( B_RS );	
	c = ascii_read_controller(  );
	*GPIO_E_MODER	 	= 0x55555555;	/* all bits outputs */
	return c;
}

void ascii_wait_ready(void)
{
	while( ( ascii_read_status() & 0x80)== 0x80 );
	delay_micro( 8 );		/* se not sid 16 */

}

void	ascii_write_char( unsigned char c)
{
	ascii_wait_ready();
	ascii_write_data( c);
	delay_micro( 39 );
}

void ascii_init()
{
	ascii_wait_ready();
	ascii_write_cmd( 0x38 );	/* Function set */
	delay_micro( 39 );
	ascii_wait_ready();
	ascii_write_cmd( 0x0C );	/* Display on */
	delay_micro( 39 );
	ascii_wait_ready();
	ascii_write_cmd( 1 );		/* Clear display */
	delay_milli( 2 );
	ascii_wait_ready();
	ascii_write_cmd( 6 );		/* Entry mode set */
	delay_micro( 39 );
}




void ascii_gotoxy( unsigned char x, unsigned char y) {
   unsigned char address;

   if(y!=1)
     address=0x40 | (x-1);
   else
     address=x-1;
   ascii_write_cmd( 0x80 | address);
}



/* ---------------------------------------------------- */
int main(int argc, char **argv)
{
	unsigned char *s ;
	unsigned  char test1[] = "Alfanumerisk ";
	unsigned  char test2[] = "Display - test";

	init_app();
	ascii_init();
	ascii_gotoxy(1,1);
	s = test1;
	while( *s )
		ascii_write_char( *s++ );
	ascii_gotoxy(1,2);
	s = test2;
	while( *s )
		ascii_write_char( *s++ );
	return 0;
}
