/*
 * 	flipflop_irq_vectored.c
 *
 * Flank triggat avbrott med vektoriserad programstyrd återställning
 * IRQ FlipFlop modul nsluts till PE0-7
 * Diodramp ansluts till PD0-7
 */
 
__attribute__((naked)) 
__attribute__((section (".start_section")) )
void startup ( void )
{
__asm__ volatile(" LDR R0,=0x2001C000\n");		
__asm__ volatile(" MOV SP,R0\n");		
__asm__ volatile(" BL main\n");		
__asm__ volatile(" B .\n");		
}


#define SYSCFG_BASE     ((volatile unsigned int *)0x40013800)
#define SYSCFG_EXTICR1  ((volatile unsigned int *)0x40013808)
#define RCC_APB2ENR     ((volatile unsigned int *)0x40023844)

#define EXTI_IMR        ((volatile unsigned int *)0x40013C00)
#define EXTI_FTSR       ((volatile unsigned int *)0x40013C0C)
#define EXTI_RTSR       ((volatile unsigned int *)0x40013C08)
#define EXTI_PR         ((volatile unsigned int *)0x40013C14)

#define EXTI3_IRQVEC    ((void (**)(void) ) 0x2001C064)
#define EXTI2_IRQVEC    ((void (**)(void) ) 0x2001C060)
#define EXTI1_IRQVEC    ((void (**)(void) ) 0x2001C05C)
#define EXTI0_IRQVEC    ((void (**)(void) ) 0x2001C058)

#define	NVIC_ISER0 		((volatile unsigned int *)0xE000E100)  

#define NVIC_EXTI3_IRQ_BPOS  (1<<9)
#define NVIC_EXTI2_IRQ_BPOS  (1<<8)
#define NVIC_EXTI1_IRQ_BPOS  (1<<7)
#define NVIC_EXTI0_IRQ_BPOS  (1<<6)

#define EXTI3_IRQ_BPOS  (1<<3)
#define EXTI2_IRQ_BPOS  (1<<2)
#define EXTI1_IRQ_BPOS  (1<<1)
#define EXTI0_IRQ_BPOS  (1<<0)


#define		PORT_BARGRAPH_BASE	0x40020C00		/* MD407 port D */
#define		PORT_FLIPFLOP_BASE	0x40021000		/* MD407 port E */

/* Flipflop */
#define		portFlipflopModer 	((volatile unsigned int *)	(PORT_FLIPFLOP_BASE))  
#define		portFlipflopOtyper 	((volatile unsigned short *)(PORT_FLIPFLOP_BASE+0x4))  
#define		portFlipflopOspeedr 	((volatile unsigned int *)	(PORT_FLIPFLOP_BASE+0x8))  
#define		portFlipflopPupdr 	((volatile unsigned int *)	(PORT_FLIPFLOP_BASE+0xC))  
#define		portFlipflopIdrLow 		((volatile unsigned char *)(PORT_FLIPFLOP_BASE+0x10))  
#define		portFlipflopOdrLow 	((volatile unsigned char *)	(PORT_FLIPFLOP_BASE+0x14))  
	
/* Bargraph */
#define		portBargraphModer 		((volatile unsigned int *)		(PORT_BARGRAPH_BASE))  
#define		portBargraphOtyper 		((volatile unsigned short *)	(PORT_BARGRAPH_BASE+0x4))  
#define		portBargraphOspeedr 	((volatile unsigned int *)		(PORT_BARGRAPH_BASE+0x8))  
#define		portBargraphOdrLow 		((volatile unsigned char *)		(PORT_BARGRAPH_BASE+0x14))  



char count;

void irq_handler_exti2 ( void )
{
 	*((unsigned int *) EXTI_PR) |= EXTI2_IRQ_BPOS;   /* Clear this interrupt */
	*portFlipflopOdrLow |= 0x40;		/* acknowledge interrupts  RST1 */   
	*portFlipflopOdrLow &= ~0x40;	/* reset acknowledge pulses */   
	if( count )
		count = 0;
	else
		count = 0xFF;
}

void irq_handler_exti1 ( void )
{
	count = 0;
	*((unsigned int *) EXTI_PR) |= EXTI1_IRQ_BPOS;   /* Clear this interrupt in EXTI */
 	*portFlipflopOdrLow |= 0x20;		/* acknowledge interrupts  RST1 */   
	*portFlipflopOdrLow &= ~0x20;	/* reset acknowledge pulses */   
}
void irq_handler_exti0 ( void )
{
	count++;
 	*((unsigned int *) EXTI_PR) |= EXTI0_IRQ_BPOS;   /* Clear this interrupt in EXTI */
	*portFlipflopOdrLow |= 0x10;		/* acknowledge interrupts  RST0 */   
	*portFlipflopOdrLow &= ~0x10;	/* reset acknowledge pulses */   
}


void appInit ( void )
{
#ifdef USBDM	
	/* starta klockor port D och E */
	* ( (unsigned long *) 0x40023830) = 0x18;
	/* starta klockor för SYSCFG */
	* ((unsigned long *)0x40023844) |= 0x4000; 	
	/* Relokera vektortabellen */
	* ((unsigned long *)0xE000ED08) = 0x2001C000;
#endif	
	/*	PORT E
 	b7-b4 will be outputs
	b3-b0 will be inputs
	*/
	*portFlipflopModer = 		0x00005500;		/* Data direction digital io */	
	*portFlipflopPupdr = 		0x00000000;		/* Input, floating */
	*portFlipflopOspeedr = 		0x0000FF00;		/* Output, high speed */
	*portFlipflopOdrLow = 0;
	
	*portFlipflopIdrLow = 0;

	*portBargraphModer	 	= 0x00005555;	/* bits 0-7 are outputs */
	*portBargraphOtyper 	= 0x00000000;	/* outputs are push/pull */	
	*portBargraphOspeedr 	= 0x00005555;	/* medium speed */

 	count = 1;
	*portFlipflopOdrLow = 0x70;	
	*portFlipflopOdrLow = ~0x70;	/* reset acknowledge pulses */   

    
   /* set PE3 as IRQ input, negative edge */
    //*((unsigned int *) RCC_APB2ENR) |= 0x4000;     /* Enable SYSCFG clock  */
	
    *SYSCFG_EXTICR1 |= 0x0444;     /* PE2..PE0->EXTI23--EXTI0  */ 
    
    /* Configure the mask bit of the interrupt line (EXTI_IMR) */
    *EXTI_IMR |= (EXTI2_IRQ_BPOS |  EXTI1_IRQ_BPOS | EXTI0_IRQ_BPOS ); 
     
    /* Configure the Trigger selection bit of the interrupt line (EXTI_RTSR and EXTI_FTSR) */
    *EXTI_RTSR |= (EXTI2_IRQ_BPOS |  EXTI1_IRQ_BPOS | EXTI0_IRQ_BPOS );		/* enable trigger on falling edge */ 
	*EXTI_FTSR &= ~(EXTI2_IRQ_BPOS |  EXTI1_IRQ_BPOS | EXTI0_IRQ_BPOS );		/* disable trigger on rising edge */ 

    /* set the interrupt vectors */
    *EXTI2_IRQVEC = irq_handler_exti2;
	*EXTI1_IRQVEC = irq_handler_exti1;
	*EXTI0_IRQVEC = irq_handler_exti0;

   
    /*  Configure the enable and mask bits that control the NVIC IRQ channel mapped to the
        external interrupt controller (EXTI) . */
    *NVIC_ISER0 |= (NVIC_EXTI2_IRQ_BPOS | NVIC_EXTI1_IRQ_BPOS | NVIC_EXTI0_IRQ_BPOS );	/* Enable these interrupts */           
 }

void main(void)
{
    appInit();
 	while(1){
        * portBargraphOdrLow = count;
	}
 }

