/*
 * 	libmd407.c
 *  includes: _crt_init, _crt_deinit, _sbrk for malloc-support.
 *  _exit
 *  template functions for stdio:  
 */
 
 
#ifdef USE_REENT
	#define _REENT_SMALL
	#include	<sys/reent.h>
	static struct _reent 	my_clib = _REENT_INIT(my_clib);
#endif
/* Include standard headers */
#include	<stdio.h>

/* Declare your functions in 'libmd407.h' */
#include	"libmd407.h"

extern DEV_DRIVER_DESC StdIn,StdOut,StdErr,KeyPad,AsciiDisplay;

/* Order is essential, must match:
 * enum {STDIN=STDIN_FILENO,STDOUT=STDOUT_FILENO,STDERR=STDERR_FILENO,KEYPAD_FILENO,ASCIIDISPLAY_FILENO};
 */
PDEV_DRIVER_DESC device_table[MAX_DEVICE] =
{
	&StdIn,
	&StdOut,
	&StdErr,
	&KeyPad,
	&AsciiDisplay,
};

 
static char *heap_end;
char * _sbrk(int incr) {
	char *prev_heap_end;
 	if (heap_end == 0) {
		heap_end = &__heap_low;
	}
	prev_heap_end = heap_end;
	if (heap_end + incr > &__heap_top) {		
		errno = ENOMEM;
			return (char *)-1;
	}
	heap_end += incr;
	return (char *) prev_heap_end;	
}


__attribute__ ( (used) )		/* Might be optimised away otherwise... */
volatile void _crt_init( int not_used) {
  char *s;
   PDEV_DRIVER_DESC fd;
   heap_end = 0;
  s = &__bss_start__;
  while( s < &__bss_end__ )
    *s++ = 0;
  for( int i = 0; i < MAX_DEVICE; i++ )
  {
    fd = device_table[i];
    if( fd && fd->init != 0)
      (void) fd->init( 0 );
   }
#ifdef USE_REENT
   _impure_ptr =  &my_clib;
   setvbuf( _stdin_r(_impure_ptr), NULL, _IONBF, 0 );
   setvbuf( _stdout_r(_impure_ptr), NULL, _IONBF, 0 );
   setvbuf( _stderr_r(_impure_ptr), NULL, _IONBF, 0 );
#else
   setvbuf( stdin, NULL, _IONBF, 0 );
   setvbuf( stdout, NULL, _IONBF, 0 );
   setvbuf( stderr, NULL, _IONBF, 0 );
 #endif
}


__attribute__ ( (used) )		/* Will be optimised away otherwise... */
volatile void _crt_deinit( int not_used ) {
  PDEV_DRIVER_DESC fd;
  for( int i = 0; i < MAX_DEVICE; i++ )
  {
    fd = device_table[i];
    if( fd && fd->deinit != 0)
      fd->deinit( 0 );
   }	
}

void _exit( int a )
{
	_crt_deinit( a );
	__asm__ volatile(".L1: B .L1\n");				/* never return */
}


/* STDIO-templates */
int _close(int file) {
      return -1;
}

int _open(const char *name, int flags, int mode)
{ 
      return -1;
}

int _fstat(int file, struct stat *st)
{ 
  PDEV_DRIVER_DESC fd;
  if( ( file < 0)||(file>= MAX_DEVICE) )
      return 0;
  fd = device_table[file];
    if( fd && fd->fstat != 0)
      return fd->fstat( st );
//	st->st_mode = S_IFCHR; 
	return 0; 
}

int _lseek(int file, int ptr, int dir)
{ 
  PDEV_DRIVER_DESC fd;
  if( ( file < 0)||(file>= MAX_DEVICE) )
      return 0;
  fd = device_table[file];
    if( fd && fd->lseek != 0)
      return fd->lseek(ptr, dir);
  return 0;
}

int _isatty(int file)
{
  PDEV_DRIVER_DESC fd;
  if( ( file < 0)||(file>= MAX_DEVICE) )
      return 0;
  fd = device_table[file];
    if( fd && fd->isatty != 0)
      return fd->isatty();
  return 0;
 }


int _write(int file, char *ptr, int len)
{
  PDEV_DRIVER_DESC fd;
  if( ( file < 0)||(file>= MAX_DEVICE) )
      return 0;
  fd = device_table[file];
    if( fd && fd->write != 0)
      return fd->write(ptr,len);
  return 0;
}

int _read(int file, char *ptr, int len)
{
  PDEV_DRIVER_DESC fd;
  if( ( file < 0)||(file>= MAX_DEVICE) )
      return 0;
  fd = device_table[file];
    if( fd && fd->read != 0)
      return fd->read(ptr,len);
  return 0;
}

