/*
 * flipflop_taskswitch.c
 * Anslut avbrottsvippa till port E0-7 
 * anslut diodramp till D0-7
 */
 
#define			STACK_SIZE	64	/* 64 * 4 bytes */
unsigned int	p0stack[STACK_SIZE];
unsigned int	p1stack[STACK_SIZE];
unsigned int	p2stack[STACK_SIZE];
unsigned int	p3stack[STACK_SIZE];

unsigned int	process_stacks[4];
int				running;


 
void startup(void) __attribute__((naked)) __attribute__((section (".start_section")) );

void startup ( void )
{
asm volatile(
	" LDR R0,=0x2001C000\n"		/* set stack */
	" MOV SP,R0\n"
	" BL main\n"				/* call main */
	".L1: B .L1\n"				/* never return */
	) ;
}

#define GPIO_D			0x40020C00
#define GPIO_MODER 		((volatile unsigned int *)	(GPIO_D))  
#define GPIO_OTYPER 	((volatile unsigned short *)(GPIO_D+0x4))  
#define GPIO_ODR_LOW	((volatile unsigned char *)	(GPIO_D+0x14))  


#define SYSCFG_BASE     ((volatile unsigned int *)0x40013800)
#define SYSCFG_EXTICR1  ((volatile unsigned int *)0x40013808)
#define RCC_APB2ENR     ((volatile unsigned int *)0x40023844)

#define EXTI_BASE       ((volatile unsigned int *)0x40013C00)
#define EXTI_IMR        ((volatile unsigned int *)0x40013C00)
#define EXTI_FTSR       ((volatile unsigned int *)0x40013C0C)
#define EXTI_RTSR       ((volatile unsigned int *)0x40013C08)
#define EXTI_PR         ((volatile unsigned int *)0x40013C14)

#define EXTI3_IRQVEC    ((volatile unsigned int *)0x2001C064)
#define	NVIC_ISER0 		((volatile unsigned int *)0xE000E100) 
#define	NVIC_ICPR0 		((volatile unsigned int *)0xE000E280)  
#define	NVIC_IPR0       ((volatile unsigned int *)0xE000E400)

#define NVIC_EXTI3_IRQ_BPOS  (1<<9)
#define EXTI3_IRQ_BPOS  (1<<3)

#define		PORT_FLIPFLOP_BASE	0x40021000		/* MD407 port E */

/* Flipflop */
#define		portFlipflopModer 	((volatile unsigned int *)	(PORT_FLIPFLOP_BASE))  
#define		portFlipflopOtyper 	((volatile unsigned short *)(PORT_FLIPFLOP_BASE+0x4))  
#define		portFlipflopOspeedr 	((volatile unsigned int *)	(PORT_FLIPFLOP_BASE+0x8))  
#define		portFlipflopPupdr 	((volatile unsigned int *)	(PORT_FLIPFLOP_BASE+0xC))  
#define		portFlipflopIdrLow 		((volatile unsigned char *)(PORT_FLIPFLOP_BASE+0x10))  
#define		portFlipflopOdrLow 	((volatile unsigned char *)	(PORT_FLIPFLOP_BASE+0x14))  
	
void	schedule( int doit )
{
	if( ! doit )
		return;
	int next = running;
	switch( running )
	{
		case	0: next = 1; break;
		case	1: next = 2; break;
		case	2: next = 3; break;
		case	3: next = 0; break;
		default:	break;
	}
	running = next;
}

int ack_irq( void )
{
	if(   *((unsigned int *) EXTI_PR) & EXTI3_IRQ_BPOS )
	{
 		*portFlipflopOdrLow = 0x70;	   
 		*portFlipflopOdrLow = ~0x70;   
		*EXTI_PR |= EXTI3_IRQ_BPOS;   /* Clear this interrupt */
		return 1;
	}
	return 0;
}


__attribute__((naked))
void 	irq_handler ( void )
{
__asm volatile(" PUSH	{R4-R7}");			
__asm volatile(" LDR	R1,=process_stacks");
__asm volatile(" LDR	R2,=running");
__asm volatile(" LDR	R2,[R2]");
__asm volatile(" LSL	R2,#2");
__asm volatile(" MOV	R0,SP");
__asm volatile(" STR	R0,[R1,R2]");
		
__asm volatile(" PUSH	{LR}");			
__asm volatile(" BL		ack_irq");
__asm volatile(" BL		schedule");
__asm volatile(" POP	{R0}");	
__asm volatile(" MOV	LR,R0");
		
__asm volatile(" LDR	R1,=process_stacks");
__asm volatile(" LDR	R2,=running");
__asm volatile(" LDR	R2,[R2]");
__asm volatile(" LSL	R2,#2");
__asm volatile(" LDR	R0,[R1,R2]");
__asm volatile(" MOV	SP,R0");
__asm volatile(" POP	{R4-R7}");			
__asm volatile(" BX		LR");
}

void flipflopInit ( void )
{
	/*	PORT E
 	b7-b4 will be outputs
	b3-b0 will be inputs
	*/
	*portFlipflopModer = 		0x00005500;		/* Data direction digital io */	
	*portFlipflopPupdr = 		0x00000000;		/* Input, floating */
	*portFlipflopOspeedr = 		0x0000FF00;		/* Output, high speed */
	*portFlipflopOdrLow = 0;
	
	*portFlipflopIdrLow = 0;

	*portFlipflopOdrLow = 0x70;	
	*portFlipflopOdrLow = ~0x70;	/* reset acknowledge pulses */   

    
   /* set PE3 as IRQ input, negative edge */
    *RCC_APB2ENR |= 0x4000;     /* Enable SYSCFG clock  */ 
    *SYSCFG_EXTICR1 |= 0x4000;     /* PE3->EXTI3  */ 
    
    /* Configure the mask bit of the interrupt line (EXTI_IMR) */
    *EXTI_IMR |= EXTI3_IRQ_BPOS; 
     
    /* Configure the Trigger selection bit of the interrupt line (EXTI_RTSR and EXTI_FTSR) */
    *EXTI_RTSR |= EXTI3_IRQ_BPOS;		/* enable trigger on falling edge */ 
	*EXTI_FTSR &= ~EXTI3_IRQ_BPOS;		/* disable trigger on rising edge */ 


    /*  Configure the enable and mask bits that control the NVIC IRQ channel mapped to the
        external interrupt controller (EXTI) . */
	*NVIC_IPR0 |= (0xf<<(3*8));   		/* interrupt priority (lowest) */
    *NVIC_ISER0 |= NVIC_EXTI3_IRQ_BPOS;	/* Enable this interrupt */           
 }


void	p0( void )
{
	while(1)
		*GPIO_ODR_LOW = 1;
}
void	p1( void )
{
	while(1)
		*GPIO_ODR_LOW = 2;
}
void	p2( void )
{
	while(1)
		*GPIO_ODR_LOW = 4;
}
void	p3( void )
{
	while(1)
		*GPIO_ODR_LOW = 8;
}

void	taskswitchInit( void )
{
/* Sätt upp initial PC och PSR */
	//#define	FP_FRAME	16*4+8		/* Flyttalsenhet aktiverad */
	#define	FP_FRAME	0		
	#define	FRAME_SIZE	(FP_FRAME+48)		/* 32 bitars ord, endast exception stack och R4-R7 */		
	int*p;
	
	process_stacks[1] = (int) p1stack + ( sizeof( p1stack) ) - FRAME_SIZE;	
	p = (int *) process_stacks[1];
	p[10] = (unsigned int) p1;
	p[11] = 0x01000000;
	
	process_stacks[2] = (int) p2stack + ( sizeof( p2stack) ) - FRAME_SIZE;
	p = (int *) process_stacks[2];
	p[10] = (unsigned int) p2;
	p[11] = 0x01000000;

	process_stacks[3] = (int) p3stack + ( sizeof( p3stack) ) - FRAME_SIZE;
	p = (int *) process_stacks[3];
	p[10] = (unsigned int) p3;
	p[11] = 0x01000000;
	running = 0;
}

void main(void)
{
    flipflopInit();
	taskswitchInit();
	*GPIO_MODER	 	= 0x00005555;
	*GPIO_OTYPER 	= 0x00000000;
    /* set the interrupt vector */
    *((void (**)(void) ) EXTI3_IRQVEC ) = irq_handler;
	p0();
	while(1){}
 }
