/*
 * 	startup.c
 *
 */
__attribute__((naked)) __attribute__((section (".start_section")) )
void startup ( void )
{
__asm__ volatile(" LDR R0,=0x2001C000\n");		/* set stack */
__asm__ volatile(" MOV SP,R0\n");
__asm__ volatile(" BL main\n");					/* call main */
__asm__ volatile(".L1: B .L1\n");				/* never return */
}



/* Port E */
#define		PORT_E_BASE		0x40021000
#define		GPIO_E_MODER 	((volatile unsigned int *)	(PORT_E_BASE))  
#define		GPIO_E_OTYPER 	((volatile unsigned short *)(PORT_E_BASE+0x4))  
#define		GPIO_E_OSPEEDR 	((volatile unsigned int *)	(PORT_E_BASE+0x8))  
#define		GPIO_E_PUPDR 	((volatile unsigned int *)	(PORT_E_BASE+0xC))  
#define		GPIO_E_IDRLOW 	((volatile unsigned char *)(PORT_E_BASE+0x10))  
#define		GPIO_E_IDRHIGH 	((volatile unsigned char *)(PORT_E_BASE+0x11))  
#define		GPIO_E_ODRLOW 	((volatile unsigned char *)	(PORT_E_BASE+0x14))  
#define		GPIO_E_ODRHIGH 	((volatile unsigned char *)	(PORT_E_BASE+0x15))  

#define 	SYSCFG_EXTICR1  ((volatile unsigned int *)0x40013808)
#define 	EXTI_IMR        ((volatile unsigned int *)0x40013C00)
#define 	EXTI_FTSR       ((volatile unsigned int *)0x40013C0C)
#define 	EXTI_RTSR       ((volatile unsigned int *)0x40013C08)
#define 	EXTI_PR         ((volatile unsigned int *)0x40013C14)
#define		NVIC_ISER0 		((volatile unsigned int *)0xE000E100)  

#define 	NVIC_EXTI3_IRQ_BPOS  (1<<9)
#define 	NVIC_EXTI2_IRQ_BPOS  (1<<8)
#define 	NVIC_EXTI1_IRQ_BPOS  (1<<7)
#define 	NVIC_EXTI0_IRQ_BPOS  (1<<6)

#define 	EXTI3_IRQ_BPOS  (1<<3)
#define	 	EXTI2_IRQ_BPOS  (1<<2)
#define 	EXTI1_IRQ_BPOS  (1<<1)
#define 	EXTI0_IRQ_BPOS  (1<<0)

#define 	EXTI2_IRQVEC    0x2001C060
#define 	EXTI1_IRQVEC    0x2001C05C
#define 	EXTI0_IRQVEC    0x2001C058



void init_gpio( void )
{
	/*	PORT E
 	b15-b8  used for hexadecimal 
	b7-b5   used for output to rows
	b4-b0 	used for input from columns, NOR gate and switches
	*/
	*GPIO_E_MODER = 0x55555400;		
	*GPIO_E_PUPDR = 0x0000A000;		/* Input, b3 and b2 pull down */
    *GPIO_E_OTYPER= 0x00000000;	    /* outputs are push/pull */
	*GPIO_E_ODRLOW = 0;
	*GPIO_E_ODRHIGH = 0;
}

static char lastkey, locked;

unsigned char get( void )
{
	unsigned short c;
	*GPIO_E_ODRLOW = 0x80 ; 
	c = *GPIO_E_IDRLOW;
	if ( c & 0x10 ){ lastkey = 0; return;}
	if ( c & 8 ){ lastkey = 1; return;}
	*GPIO_E_ODRLOW = 0x40 ; 
	c = *GPIO_E_IDRLOW;
	if ( c & 0x10 ){ lastkey = 2; return;}
	if ( c & 8 ){ lastkey = 3; return;}
	*GPIO_E_ODRLOW = 0x20 ; 
	c = *GPIO_E_IDRLOW;
	if ( c & 0x10 ){ lastkey = 4; return;}
	if ( c & 8 ){ lastkey = 5; return;}
}


void KeybIRQ ( void )
{
 	*EXTI_PR = EXTI0_IRQ_BPOS;   /* Clear this interrupt */
	lastkey = get();
}
void S1IRQ ( void )
{
 	*EXTI_PR = EXTI1_IRQ_BPOS;   /* Clear this interrupt */
	locked = 0;;
}
void S2IRQ ( void )
{
 	*EXTI_PR = EXTI2_IRQ_BPOS;   /* Clear this interrupt */
	locked = 1;;
}



void init_exti( void )
{
	*SYSCFG_EXTICR1|= 0x0444;     /* PE2..PE0->EXTI23--EXTI0  */ 
	/* Configure the mask bit of the interrupt line (EXTI_IMR) */
    *EXTI_IMR |= (EXTI2_IRQ_BPOS |  EXTI1_IRQ_BPOS | EXTI0_IRQ_BPOS );   
    /* Configure the Trigger selection bit of the interrupt line (EXTI_RTSR and EXTI_FTSR) */
    *EXTI_FTSR |= (EXTI1_IRQ_BPOS | EXTI0_IRQ_BPOS );		/* enable trigger on falling edge */ 
	*EXTI_RTSR &= ~(EXTI1_IRQ_BPOS | EXTI0_IRQ_BPOS );		/* disable trigger on rising edge */ 
	*EXTI_RTSR |= EXTI2_IRQ_BPOS ;		/* enable trigger on rising edge */ 
	*EXTI_FTSR &= ~EXTI2_IRQ_BPOS;		/* disable trigger on falling edge */ 
    /* set the interrupt vectors */
	*((void (**)(void) ) EXTI0_IRQVEC)  = KeybIRQ;
 	*((void (**)(void) ) EXTI1_IRQVEC)  = S1IRQ;
	*((void (**)(void) ) EXTI2_IRQVEC)  = S2IRQ;
    /* Enable these interrupts */
    *NVIC_ISER0 |= (NVIC_EXTI2_IRQ_BPOS | NVIC_EXTI1_IRQ_BPOS | NVIC_EXTI0_IRQ_BPOS );	           
}



void main(void)
{
	init_gpio();
	init_exti();
	locked = 1;
	while( 1 )
	{
		if( locked ) *GPIO_E_ODRHIGH = 0xF;
		else *GPIO_E_ODRHIGH = lastkey;
	}
}

