/*
 * 	startup.c
 *
 */
__attribute__((naked)) __attribute__((section (".start_section")) )
void startup ( void )
{
__asm__ volatile(" LDR R0,=0x2001C000\n");		/* set stack */
__asm__ volatile(" MOV SP,R0\n");
__asm__ volatile(" BL main\n");					/* call main */
__asm__ volatile(".L1: B .L1\n");				/* never return */
}


/* ---------------------------------------------------- */
/* Port D */
#define		PORT_D_BASE		0x40020C00
#define		GPIO_D_MODER 	((volatile unsigned int *)	(PORT_D_BASE))  
#define		GPIO_D_OTYPER 	((volatile unsigned short *)(PORT_D_BASE+0x4))  
#define		GPIO_D_OSPEEDR 	((volatile unsigned int *)	(PORT_D_BASE+0x8))  
#define		GPIO_D_PUPDR 	((volatile unsigned int *)	(PORT_D_BASE+0xC))  
#define		GPIO_D_IDRLOW 	((volatile unsigned char *)(PORT_D_BASE+0x10))  
#define		GPIO_D_IDRHIGH 	((volatile unsigned char *)(PORT_D_BASE+0x11))  
#define		GPIO_D_ODRLOW 	((volatile unsigned char *)	(PORT_D_BASE+0x14))  
#define		GPIO_D_ODRHIGH 	((volatile unsigned char *)	(PORT_D_BASE+0x15))  


void init( void )
{
	/*	PORT D
 	b15-b12 used for output to rows
	b11-b8  used for input from columns
	b7-b0 	used for output to display
	*/
	*GPIO_D_MODER = 0x55005555;		
	*GPIO_D_PUPDR = 0x00AA0000;		/* Input, pull down */
    *GPIO_D_OTYPER= 0x00000000;	    /* outputs are push/pull */

	*GPIO_D_ODRLOW = 0;
	*GPIO_D_ODRHIGH = 0;	
}

void kbdActivate( unsigned int row )
{
	switch( row )
	{
		case	1: *GPIO_D_ODRHIGH = 0x10 ; 	break;
		case 	2: *GPIO_D_ODRHIGH = 0x20 ; 	break;
		case	3: *GPIO_D_ODRHIGH = 0x40 ; 	break;
		case	4: *GPIO_D_ODRHIGH = 0x80 ; 	break;
		case	5: *GPIO_D_ODRHIGH = 0xF0 ; 	break;
		case	0: *GPIO_D_ODRHIGH = 0x00; 	break;
	}
}

int kbdGetCol ( void )
{
	unsigned short c;
	c = *GPIO_D_IDRHIGH;
 	if ( c & 0x8 )		return 4;
	if ( c & 0x4 ) 	    return 3;
	if ( c & 0x2 ) 	    return 2;
	if ( c & 0x1 ) 	    return 1;
	return 0;
}


unsigned char keyb(void) 
{
	unsigned char key[]={1,2,3,0xA,4,5,6,0xB,7,8,9,0xC,0xE,0,0xF,0xD};
	int row, col;
	for (row=1; row <=4 ; row++ ) {
		kbdActivate( row );	
		if( (col = kbdGetCol () ) )
		{
			return key [4*(row-1)+(col-1) ];
		}
	}
	*GPIO_D_ODRHIGH = 0;
	return 0xFF;
}

char keycode(void)
{
    char c;
    while( (c = keyb() )==0xFF);
    while( keyb() != 0xFF);
    return c;
}

char	bitconvert(char method, char value)
{
	char rv;
	switch( method )
	{
		case 0xA: rv = value; break;
		case 0xB: rv = ~value ; break;  
		case 0xC: rv = ( value >> 1 ); break;
		case 0xD: rv = ( value << 1 ); break;
        default: rv = 0;
	}
	return rv;
}


char get_valid_value( void )
{
    char c1, c2;
    do{
        c1 = keycode();
    }while( c1 >= 10);
    do{
        c2 = keycode();
    }while( c2 >= 10);
    
    return (c1 << 4) | c2;
}


char get_valid_method( void )
{
    char c;
    do{
        c = keycode();
    }while( (c < 10) || (c==14) || (c==15) );
    return c;
}

void main(void)
{
    char value, method;
    init();
    while( 1 )
    {
        value = get_valid_value();
        method = get_valid_method();
        *GPIO_D_ODRLOW = bitconvert(method, value);
    }
}

