/*
 * 	startup.c
 *
 */
__attribute__((naked)) __attribute__((section (".start_section")) )
void startup ( void )
{
__asm__ volatile(" LDR R0,=0x2001C000\n");		/* set stack */
__asm__ volatile(" MOV SP,R0\n");
__asm__ volatile(" BL main\n");					/* call main */
__asm__ volatile(".L1: B .L1\n");				/* never return */
}

#define		PORT_D_BASE		0x40020C00
#define		GPIO_D_MODER 	((volatile unsigned int *)	(PORT_D_BASE))  
#define		GPIO_D_OTYPER 	((volatile unsigned short *)(PORT_D_BASE+0x4))  
#define		GPIO_D_OSPEEDR 	((volatile unsigned int *)	(PORT_D_BASE+0x8))  
#define		GPIO_D_PUPDR 	((volatile unsigned int *)	(PORT_D_BASE+0xC))  
#define		GPIO_D_IDRLOW 	((volatile unsigned char *)(PORT_D_BASE+0x10))  
#define		GPIO_D_IDRHIGH 	((volatile unsigned char *)(PORT_D_BASE+0x11))  
#define		GPIO_D_ODRLOW 	((volatile unsigned char *)	(PORT_D_BASE+0x14))  
#define GPIO_D_ODRHIGH 	((volatile unsigned char *)	(PORT_D_BASE+0x15))  


#define STK_CTRL  	((volatile unsigned int *)(0xE000E010))  
#define STK_LOAD  	((volatile unsigned int *)(0xE000E014))  
#define STK_VAL  	((volatile unsigned int *)(0xE000E018))  


#define SYSTICK_IRQVEC  ((volatile unsigned int *)0x2001C03C)


void init( void )
{
	/*	PORT D
 	b15-b12 used for output to rows
	b11-b8  used for input from columns
	b7-b0 	used for output to 7-seg display
	*/
	*GPIO_D_MODER = 0x55005555;		
	*GPIO_D_PUPDR = 0x00AA0000;		/* Input, pull down */
    *GPIO_D_OTYPER= 0x00000000;	    /* outputs are push/pull */
}


static void kbdActivate( unsigned int row )
{
	switch( row )
	{
		case	1: *GPIO_D_ODRHIGH = 0x10 ; 	break;
		case 	2: *GPIO_D_ODRHIGH = 0x20 ; 	break;
		case	3: *GPIO_D_ODRHIGH = 0x40 ; 	break;
		case	4: *GPIO_D_ODRHIGH = 0x80 ; 	break;
		case	0: *GPIO_D_ODRHIGH = 0x00; 	break;
	}
}

static int kbdGetCol ( void )
{
	unsigned short c;
	c = *GPIO_D_IDRHIGH;
 	if ( c & 0x8 )		return 4;
	if ( c & 0x4 ) 	    return 3;
	if ( c & 0x2 ) 	    return 2;
	if ( c & 0x1 ) 	    return 1;
	return 0;
}


unsigned char keyb(void) 
{
	unsigned char key[]={1,2,3,0xA,4,5,6,0xB,7,8,9,0xC,0xE,0,0xF,0xD};
	int row, col;
	for (row=1; row <=4 ; row++ ) {
		kbdActivate( row );	
		if( (col = kbdGetCol () ) )
		{
			return key [4*(row-1)+(col-1) ];
		}
	}
	*GPIO_D_ODRHIGH = 0;
	return 0xFF;
}

void	outseg( char c )
{
static char segCode[]={ 
	0x3F,0x06,0x5B,0x4F,0x66,0x6D,0x7D,0x07,
	0x7F,0x67,0x77,0x7C,0x39,0x5E,0x79,0x71, 0x40 };
	
	
	if( c > 16){
		*GPIO_D_ODRLOW = 0;
		return;
	}
	*GPIO_D_ODRLOW = segCode[c];
}


void blocked_delay_10ms( void )
{
	/* SystemCoreClock = 168000000 */
	*STK_CTRL = 0;
//	*STK_LOAD = ( 1680000-1 ); /* 10 ms */
	*STK_LOAD = ( 168-1 );	/* Simulator value */
	*STK_VAL = 0;
	*STK_CTRL = 5;
	while( (*STK_CTRL & 0x10000)==0);
}

void blocked_delay( unsigned int ms )
{
	while( ms/10 ){
		blocked_delay_10ms();
		ms -=10;
	}
}

static int timeout;
void	systick_irq_handler( void );
void start_timeout_period( void )
{
	/* SystemCoreClock = 168000000 */
	*SYSTICK_IRQVEC = systick_irq_handler;
	*STK_CTRL = 0;
//	*STK_LOAD = ( 1680000-1 );
	*STK_LOAD = ( 1680-1 );	/* Simulator value */
	*STK_VAL = 0;
	*STK_CTRL = 7;
}

void	systick_irq_handler( void )
{
	if( timeout ){
		timeout--;
		start_timeout_period( );
	}
}

void start_timeout( void )
{
	timeout = 500; /* 100 * 10 ms = 1 sec */
	start_timeout_period( );
}

char random( void )
{
	static int seed = 111;
	seed = 147*seed;
	return seed & 0x7F;
}


void display3sec( char c )
{
	for( int i = 0; i<3;i++)
	{
		outseg(0xFF); 
		blocked_delay( 500 );
		outseg(c); 
		blocked_delay( 500 );
	}
	
}

void main(void)
{
	char tmp; 
	init();
	while(1)
	{
		unsigned int t = random() >> 4;
		t++;
		blocked_delay( t * 1000 );
		t = random() >> 3;
		outseg( (char) t );
		start_timeout();
		do{
			tmp = keyb();
			if( t == tmp )
				break;
		}while (timeout );
		if( timeout )
		{
			display3sec( tmp );
		}else{
			display3sec( 16 );			
			break;
		}
	}
}

