/*
 * 	square_wave_deadtime.c
 *
 */
__attribute__((naked)) __attribute__((section (".start_section")) )
void startup ( void )
{
__asm__ volatile(" LDR R0,=0x2001C000\n");		/* set stack */
__asm__ volatile(" MOV SP,R0\n");
__asm__ volatile(" BL main\n");					/* call main */
__asm__ volatile(".L1: B .L1\n");				/* never return */
}

#define PORT_E_BASE		0x40021000
#define GPIO_E_MODER 	((volatile unsigned int *)	(PORT_E_BASE))  
#define GPIO_E_OTYPER 	((volatile unsigned short *)(PORT_E_BASE+0x4))  
#define GPIO_E_OSPEEDR 	((volatile unsigned int *)	(PORT_E_BASE+0x8))  
#define GPIO_E_PUPDR 	((volatile unsigned int *)	(PORT_E_BASE+0xC))  
#define GPIO_E_IDRLOW 	((volatile unsigned char *)(PORT_E_BASE+0x10))  
#define GPIO_E_IDRHIGH 	((volatile unsigned char *)(PORT_E_BASE+0x11))  
#define GPIO_E_ODRLOW 	((volatile unsigned char *)	(PORT_E_BASE+0x14))  
#define GPIO_E_ODRHIGH 	((volatile unsigned char *)	(PORT_E_BASE+0x15))  

#define STK_CTRL  		((volatile unsigned int *)(0xE000E010))  
#define STK_LOAD  		((volatile unsigned int *)(0xE000E014))  
#define STK_VAL  		((volatile unsigned int *)(0xE000E018))  

#define setbit(x)   ( *GPIO_E_ODRLOW |= (1<<x) )
#define clearbit(x) ( *GPIO_E_ODRLOW &= ~(1<<x))

void init( void )
{
  /* Only b3-b0 are used here,
   * b4-b7 used bya other application,
   * hence Read/Modify/Write operations
   */
  *GPIO_E_MODER  &= 0xFFFFFF00;
  *GPIO_E_MODER  |= 0x00000050;
  *GPIO_E_PUPDR  &= 0xFFFFFF00;
  *GPIO_E_PUPDR  |= 0x00000055;
  *GPIO_E_OTYPER &= 0xFFFFFFF3;
  
  setbit(3);
  clearbit(2);
}


void delay_100_micro( void )
{
	/* SystemCoreClock = 168000000
     * blocking delay
     */
	*STK_CTRL = 0;
	*STK_LOAD = ( 16800-1 );	/* 100 us */
	*STK_VAL = 0;
	*STK_CTRL = 5;
	while( (*STK_CTRL & 0x10000)==0);
}

void delay_ms( unsigned int ms )
{
    /* Number of 100 usecs delays
     * are obtained from:
     * (ms * 10), assuming ms > 0
     */
    unsigned int delay = ms*10;
	while( delay ){
		delay_100_micro();
		delay--;
	}
}

void delay_shortpulse( unsigned int w )
{
    /* Number of 100 usecs delays
     * are obtained from:
     * (w *10) - 2, assuming w > 0
     */
    unsigned int delay = (w*10)-2;
	while( delay ){
		delay_100_micro();
		delay--;
	}
}


void main(void)
{
    unsigned int w,p;
    init();
    while( 1 )
    {
        switch( *GPIO_E_IDRLOW & 3 )
        {
            case 0: w = 2; p = 10; break;
            case 1: w = 4; p = 10; break;
            case 2: w = 20; p = 100; break;
            default: w = 40; p = 100; break;
        }
        setbit( 2 );
        delay_100_micro();
        clearbit(3);
        delay_shortpulse( w );
        setbit(3);
        delay_100_micro();
        clearbit(2);
        delay_ms( p-w );
    }
}

