/*
 * 	exam_time_system.c
 *  PE0-7 : DIL-switch
 *  PD15-8: 7-segment display
 *  PD0-7 : 7-segment display
 */
__attribute__((naked)) __attribute__((section (".start_section")) )
void startup ( void )
{
__asm volatile(
	" LDR R0,=0x2001C000\n"		/* set stack */
	" MOV SP,R0\n"
	" BL main\n"				/* call main */
	"_exit: B .\n"				/* never return */
	) ;
}


#define GPIO_D			0x40020C00
#define GPIO_D_MODER 	((volatile unsigned int *)	(GPIO_D))  
#define GPIO_D_OTYPER 	((volatile unsigned short *)(GPIO_D+0x4))  
#define GPIO_D_ODR		((volatile unsigned short *)	(GPIO_D+0x14))  

#define GPIO_E			0x40021000
#define GPIO_E_MODER 	((volatile unsigned int *)	(GPIO_E))  
#define GPIO_E_PUPDR 	((volatile unsigned int *)	(GPIO_E+0xC))  
#define GPIO_E_IDR_LOW	((volatile unsigned char *)	(GPIO_E+0x10))  


#define SYSCFG_EXTICR1  ((volatile unsigned int *)0x40013808)

#define EXTI_IMR        ((volatile unsigned int *)0x40013C00)
#define EXTI_FTSR       ((volatile unsigned int *)0x40013C0C)
#define EXTI_RTSR       ((volatile unsigned int *)0x40013C08)
#define EXTI_PR         ((volatile unsigned int *)0x40013C14)

#define	NVIC_ISER0 		((volatile unsigned int *)0xE000E100) 


#define EXTI0_IRQVEC    ((volatile unsigned int *)0x2001C058)
#define EXTI1_IRQVEC    ((volatile unsigned int *)0x2001C05C)
#define EXTI2_IRQVEC    ((volatile unsigned int *)0x2001C060)

#define STK_CTRL  	((volatile unsigned int *)(0xE000E010))  
#define STK_LOAD  	((volatile unsigned int *)(0xE000E014))  
#define STK_VAL  	((volatile unsigned int *)(0xE000E018))  


#define SYSTICK_IRQVEC  ((volatile unsigned int *)0x2001C03C)


void	exti0_irq_handler( void );
void	exti1_irq_handler( void );
void	exti2_irq_handler( void );


void portInit ( void )
{
	*GPIO_E_MODER	= 0;
	*GPIO_E_PUPDR = 0;
	*GPIO_D_MODER	= 0x55555555;
	*GPIO_D_OTYPER 	= 0x00000000;
 }


void extiInit ( void )
{
     *SYSCFG_EXTICR1 |= 0x0444;     /* PE0->EXTI0, PE1->EXTI1,PE2->EXTI2 */ 
    
    /* Configure the mask bit of the interrupt line (EXTI_IMR) */
    *EXTI_IMR |= (1<<0)|(1<<1)|(1<<2); 
     
    /* Configure the Trigger selection bit of the interrupt line (EXTI_RTSR and EXTI_FTSR) */
    *EXTI_RTSR |= (1<<0)|(1<<1)|(1<<2);		/* enable trigger on falling edge */ 
	*EXTI_FTSR |= (1<<0)|(1<<1)|(1<<2);		/* enable trigger on rising edge */ 
    *NVIC_ISER0 |= (1<<6)|(1<<7)|(1<<8);

	*EXTI0_IRQVEC = exti0_irq_handler;
	*EXTI1_IRQVEC = exti1_irq_handler;
	*EXTI2_IRQVEC = exti2_irq_handler;	          
 }





static unsigned char Clock1running, Clock2running, Clock3running;
static unsigned short Clock1, Clock2, Clock3;

void	systick_irq_handler( void )
{
	if( Clock1running ) Clock1++;
	if( Clock2running ) Clock2++;
	if( Clock3running ) Clock3++;
}

void systickInit( void )
{
	/* SystemCoreClock = 168000000 */
	*SYSTICK_IRQVEC = systick_irq_handler;

	*STK_CTRL = 0;
//	*STK_LOAD = ( 1680000-1 );
	*STK_LOAD = ( 168-1 );	/* Simulator value */
	*STK_VAL = 0;
	*STK_CTRL = 7;
}

void	exti0_irq_handler( void )
{
	if(   *((unsigned int *) EXTI_PR) & 1 )
	{
		*EXTI_PR |= 1; 
		if( *GPIO_E_IDR_LOW & 1) Clock1running = 1; else Clock1running = 0;
	}
}

void	exti1_irq_handler( void )
{
	if(   *((unsigned int *) EXTI_PR) & 2 )
	{
		*EXTI_PR |= 2; 
		if( *GPIO_E_IDR_LOW & 2) Clock2running = 1; else Clock2running = 0;
	}
}
void	exti2_irq_handler( void )
{
	if(   *((unsigned int *) EXTI_PR) & 4 )
	{
		*EXTI_PR |= 4; 
		if( *GPIO_E_IDR_LOW & 4) Clock3running = 1; else Clock3running = 0;
	}
}


void main(void)
{
	portInit();
	extiInit();
	systickInit();
	while( 1 )
	{
		switch( *GPIO_E_IDR_LOW >> 6 )
		{
			case 0: *GPIO_D_ODR = 0; break;
			case 1:
					if(*GPIO_E_IDR_LOW & (1<<3) ) /* RESET */
					{
						Clock1running = 0;
						Clock1 = 0;
					}
					*GPIO_D_ODR = Clock1 ; break;
			case 2: 
					if(*GPIO_E_IDR_LOW & (1<<4) ) /* RESET */
					{
						Clock2running = 0;
						Clock2 = 0;
					}
					*GPIO_D_ODR = Clock2; break;
			case 3: 
					if(*GPIO_E_IDR_LOW & (1<<5) ) /* RESET */
					{
						Clock3running = 0;
						Clock3 = 0;
					}
					*GPIO_D_ODR = Clock3; break;
		}
	}
}

