/*
 * 	startup.c
 *
 */
 /* Include standard headers */
#include <stdio.h>
#include <errno.h>
#include <sys/stat.h>
#include <unistd.h>

typedef struct
{
  char name[16];	
  int (*init) (int);
  void (*deinit) (int);
  int (*fstat)(struct stat *st);
  int (*isatty)(void);
  int (*open)(const char name,int flags,int mode);
  int (*close)(void);
  int (*lseek)(int ptr, int dir);
  int (*write)(char *ptr, int len);
  int (*read)(char *ptr, int len); 
} DEV_DRIVER_DESC, *PDEV_DRIVER_DESC;


/* Constants */
#define MAX_DEVICE  3

/* File handle constants */
enum {KEYPAD_FILENO=3,ASCIIDISPLAY_FILENO};


/* Constant defined by linker */
extern char __heap_low; 
extern char __heap_top; 
extern char __bss_start__; 
extern char __bss_end__; 

/* Library defined functions */
void crt_init( void );
void crt_deinit( void );

void *_sbrk(int);

extern DEV_DRIVER_DESC StdIn,StdOut,StdErr;
 
 PDEV_DRIVER_DESC device_table[MAX_DEVICE] =
{
	&StdIn,
	&StdOut,
	&StdErr,
};

__attribute__((naked)) __attribute__((section (".start_section")) )
void startup ( void )
{
__asm__ volatile(" MOVE #0x4000,%A7\n");		/* set stack */
__asm__ volatile(" JSR    _crt_init\n");					/* call main */
__asm__ volatile(" JSR    main\n");					/* call main */
__asm__ volatile(" JSR    _crt_deinit\n");					/* call main */
__asm__ volatile(" BRA _exit\n");				/* never return */
}


/* Device code... */
#define	DUART_BASE	0x3E0801	/* (MC68681, odd data bus) */
typedef	unsigned char *	PORT;

static char	_tstchar( void )
{
	/* if there is a character at DUART, return it
	 * else return 0
	 */
	if ( *((PORT)(DUART_BASE+2)) & 1 ) /* if RDRF==1 */
		return (int) ( *((PORT)(DUART_BASE+6))  );
	return 0;
}

static void	_outchar( char c )
{
	/* output character to DUART */
	while( ( *((PORT)(DUART_BASE+2)) & 4 )== 0);	/* spin for TDRE==1 */
	( *((PORT)(DUART_BASE+6)) ) = (unsigned char) c;
}

static int usart_read(char *ptr, int len)
{
    char c;
	int items = 0;
    while( 1 )
    {
		c = _tstchar();
		if( c != 0)
        {
            *ptr++ = c;
             items++;
        }
		if( len == items)
			return items;
    }
}





static int usart_write(char *ptr, int len)
{
    int rval = len;
 	while (len--)
      _outchar(*ptr++);
	return rval;
}

static int usart_isatty(void)
{
		return 1;
}

static int usart_init( int initval )
{
		/* DUART */
	*((PORT)(DUART_BASE)) = 0x13;
	*((PORT)(DUART_BASE+2)) = 0xBB;
	*((PORT)(DUART_BASE+4)) = 5;		

     return 1; /* Ok init */
}


static void usart_deinit( int deinitval)
{
	fflush(stdout); /* Will cause terminal flush... */
    /* Disable USART and interrupts */
}



DEV_DRIVER_DESC StdIn = 
{
  {"stdin"},
  usart_init,
  usart_deinit,
  0,
  usart_isatty,
  0,
  0,
  0,
  0,
  usart_read
};

DEV_DRIVER_DESC StdOut = 
{
  {"stdout"},
  usart_init,
  usart_deinit,
  0,
  usart_isatty,
  0,
  0,
  0,
  usart_write,
  0
};
DEV_DRIVER_DESC StdErr = 
{
  {"stderr"},
  usart_init,
  usart_deinit,
  0,
  usart_isatty,
  0,
  0,
  0,
  usart_write,
  0
};





/* End device code */

static char *heap_end;
void * sbrk(ptrdiff_t incr) {
	char *prev_heap_end;
 	if (heap_end == 0) {
		heap_end = &__heap_low;
	}
	prev_heap_end = heap_end;
	if (heap_end + incr > &__heap_top) {		
		errno = ENOMEM;
			return (char *)-1;
	}
	heap_end += incr;
	return (char *) prev_heap_end;	
}


__attribute__ ( (used) )		/* Might be optimised away otherwise... */
volatile void _crt_init( int not_used) {
  char *s;
   PDEV_DRIVER_DESC fd;
   heap_end = 0;
  s = &__bss_start__;
  while( s < &__bss_end__ )
    *s++ = 0;
  for( int i = 0; i < MAX_DEVICE; i++ )
  {
    fd = device_table[i];
    if( fd && fd->init != 0)
      (void) fd->init( 0 );
   }
#ifdef USE_REENT
   _impure_ptr =  &my_clib;
   setvbuf( _stdin_r(_impure_ptr), NULL, _IONBF, 0 );
   setvbuf( _stdout_r(_impure_ptr), NULL, _IONBF, 0 );
   setvbuf( _stderr_r(_impure_ptr), NULL, _IONBF, 0 );
#else
   setvbuf( stdin, NULL, _IONBF, 0 );
   setvbuf( stdout, NULL, _IONBF, 0 );
   setvbuf( stderr, NULL, _IONBF, 0 );
 #endif
}


__attribute__ ( (used) )		/* Will be optimised away otherwise... */
volatile void _crt_deinit( int not_used ) {
  PDEV_DRIVER_DESC fd;
  for( int i = 0; i < MAX_DEVICE; i++ )
  {
    fd = device_table[i];
    if( fd && fd->deinit != 0)
      fd->deinit( 0 );
   }	
}

void _exit( int a )
{
	_crt_deinit( a );
	__asm__ volatile(".L1: BRA .L1\n");				/* never return */
}


/* STDIO-templates */
int close(int file) {
      return -1;
}

int open(const char *name, int flags, int mode)
{ 
      return -1;
}

int fstat(int file, struct stat *st)
{ 
  PDEV_DRIVER_DESC fd;
  if( ( file < 0)||(file>= MAX_DEVICE) )
      return 0;
  fd = device_table[file];
    if( fd && fd->fstat != 0)
      return fd->fstat( st );
//	st->st_mode = S_IFCHR; 
	return 0; 
}

off_t lseek(int file, off_t ptr, int dir)
{ 
  PDEV_DRIVER_DESC fd;
  if( ( file < 0)||(file>= MAX_DEVICE) )
      return 0;
  fd = device_table[file];
    if( fd && fd->lseek != 0)
      return fd->lseek(ptr, dir);
  return 0;
}

int isatty(int file)
{
  PDEV_DRIVER_DESC fd;
  if( ( file < 0)||(file>= MAX_DEVICE) )
      return 0;
  fd = device_table[file];
    if( fd && fd->isatty != 0)
      return fd->isatty();
  return 0;
 }


int write(int file, const void *ptr, size_t len)
{
  PDEV_DRIVER_DESC fd;
  if( ( file < 0)||(file>= MAX_DEVICE) )
      return 0;
  fd = device_table[file];
    if( fd && fd->write != 0)
      return fd->write(ptr,len);
  return 0;
}

int read(int file, void *ptr, size_t len)
{
  PDEV_DRIVER_DESC fd;
  if( ( file < 0)||(file>= MAX_DEVICE) )
      return 0;
  fd = device_table[file];
    if( fd && fd->read != 0)
      return fd->read(ptr,len);
  return 0;
}


void main(void)
{
	printf( "Hello World");
}

