/*
 * 	startup.c
 *
 */
 #include <sys/time.h>
 #include <time.h>
 #include <stdio.h>


 /* Funktionen _times är plattformsberoende 
 * och vi definierar den här för MD407
*/
 
struct tms {
clock_t tms_utime;  /* user time */
clock_t tms_stime;  /* system time */
clock_t tms_cutime; /* user time of children */
clock_t tms_cstime; /* system time of children */
};

 
 #include <libMD407.h>
  
__attribute__((naked)) __attribute__((section (".start_section")) )
void startup ( void )
{
__asm__ volatile(" LDR R0,=__stack_top\n");		/* set stack */
__asm__ volatile(" MOV SP,R0\n");
__asm__ volatile(" BL _crt_init\n");			/* init C-runtime library */
__asm__ volatile(" BL main\n");					/* call main */
__asm__ volatile(" BL _crt_deinit\n");			/* deinit C-runtime library */
__asm__ volatile(" B  .-4\n");	
}
 
 

static uint64_t current_time;

/* Timer 6 */
#define		TIM6_CR1 	((volatile unsigned short *)	0x40001000)  
#define		TIM6_CR2 	((volatile unsigned short *)	0x40001004) 
#define		TIM6_DIER 	((volatile unsigned short *)	0x4000100C)  
#define		TIM6_SR 	((volatile unsigned short *)	0x40001010)  
#define		TIM6_EGR 	((volatile unsigned short *)	0x40001014) 
#define		TIM6_CNT 	((volatile unsigned short *)	0x40001024)  
#define		TIM6_PSC	((volatile unsigned short *)	0x40001028)  
#define		TIM6_ARR	((volatile unsigned short *)	0x4000102C)  

#define NVIC_TIM6_IRQ_BPOS  (1<<22)
#define	NVIC_TIM6_ISER 		((volatile unsigned int *)	0xE000E104)  
#define	TIM6_IRQVEC 		((void (**)(void) ) 0x2001C118 ) 


#define UIF		(1<<0)
#define UIE		(1<<0)
#define UG		(1<<0)
#define ARPE	(1<<7)
#define OPM		(1<<3)
#define URS		(1<<2)
#define UDIS	(1<<1)
#define	CEN		(1<<0)


volatile int ticks;
volatile int seconds;


#define SIMULATOR

void crt_timer_interrupt( void )
{  /* 100 ms period */
__asm__ volatile(" cpsid 	i\n");		/* set I=1 */
	*TIM6_SR &= ~UIF;
#ifdef SIMULATOR
	current_time++;
#else
	current_time++;
#endif
__asm__ volatile(" cpsie 	i\n");		/* set I=0 */
}

/* Reset and Clock Generator */
#define		RCC_APB1ENR 	((volatile unsigned int *)	0x40023840)  
#define		RCC_APB1RSTR 	((volatile unsigned int *)	0x40023820)  

void crt_timer_init( )
{
	/* System core clock: 168MHz,  
	 * * TIM6 and TIM7 clocks: 84 MHz
	 * APB2 Clock: 84 MHz
	 * APB1 Clock: 42 MHz */
	 
	/* For Now, move to debugger MD407 and simulator init */ 
	*RCC_APB1ENR	|= 0x10; 	/* Enable TIMER 6 */	
	*RCC_APB1RSTR	|= 0x10; 	/* Reset TIMER 6 */	
	*RCC_APB1RSTR	&= ~0x10; 	
	 
	 
	 
	 
	*TIM6_CR1 &= ~CEN;	 	 
	*TIM6_IRQVEC = crt_timer_interrupt;
	*NVIC_TIM6_ISER |= NVIC_TIM6_IRQ_BPOS;	/* Enable this interrupt */ 
	 
	 /* 10 ms */
#ifdef SIMULATOR
	*TIM6_PSC= 839; 
	*TIM6_ARR = 999;
#else
	*TIM6_PSC= 839; 
	*TIM6_ARR = 999;
#endif
	*TIM6_DIER |= UIE;
	*TIM6_CR1 |= CEN;
}

#define SIMULATOR



void crt_timer_deinit( )
{
	 *TIM6_CR1 &= ~CEN;
}


 
clock_t _times(struct tms *buf) {
    buf->tms_utime = buf->tms_stime = current_time;
    return current_time;
}


int _gettimeofday( struct timeval *tv, void *tzvp )
{
    uint64_t t = current_time;  // get uptime in milliseconds
    tv->tv_sec = t / 100;  // convert to seconds
    tv->tv_usec = ( t % 100 );  // get remaining microseconds
    return 0;  // return non-zero for error
} // end _gettimeofday()


/*
 * TODO:
 * Koden fungerar som avsett i MD407HW
 * i simulatorn kan det gå en stund därefter urspårning
 * verkar ha att göra med den interna hanteringen av avbrott.
 * Kolla speciellt hur timern uppdateras för olika "RunModes"
 */

void main(void)
{
	char buf[64];
	time_t seconds, save;
	int c;
	struct tm *tp;
	char *s;
	
   crt_timer_init();
	printf("Startar\n");
	while( 1 )
	{
			seconds = time(NULL);
			if( seconds != save ){
				tp = gmtime(&seconds);
				s = asctime(tp);
				printf( "%s", s);
				save = seconds;
			}
		}
	
    crt_timer_deinit();
}

