/*
 * 	startup.c
 *  Testkod för simulator "graphic display"
 *  Geometrisk figurer som roterar kring "egen tyngdpunkt"
 *  Avsedd för minsta displaystorlek 160x140
 */
__attribute__((naked)) __attribute__((section (".start_section")) )
void startup ( void )
{
__asm__ volatile(" LDR R0,=0x2001C000\n");		/* set stack */
__asm__ volatile(" MOV SP,R0\n");
__asm__ volatile(" BL main\n");					/* call main */
__asm__ volatile(".L1: B .L1\n");				/* never return */
}

typedef struct gamedisplay
{
	unsigned int ex:1;
	unsigned int :1;
	unsigned int cmd:5;
	unsigned int rot:9;

	unsigned int :0;
	unsigned int blue:8;
	unsigned int green:8;
	unsigned int red:8;
	unsigned int bgint:8;

	unsigned int :0;
	unsigned int xcoord1:10;
	unsigned int :6;
	unsigned int ycoord1:10;
	
	unsigned int :0;
	unsigned int xcoord2:10;
	unsigned int :6;
	unsigned int ycoord2:10;
	
	unsigned int :0;
	unsigned int xcoord3:10;
	unsigned int :6;
	unsigned int ycoord3:10;

} GAMEDISPLAY;

#define	R	(( GAMEDISPLAY *) 0xF0000000)


void setbgnd( int intensity )
{
	R->bgint = intensity;	
	R->cmd = 0;
	R->ex = 0;
	R->ex = 1;
}

void rect(int x,int y, int xwidth, int ywidth, int draw , int filled )
{
	R->xcoord1 = x;
	R->ycoord1 = y;
	R->xcoord2 = xwidth+x;
	R->ycoord2 = ywidth+y;
	if( filled )
	{
		if( draw )
			R->cmd = 8;
		else
			R->cmd = 9;
		
	}else{
		if( draw )
			R->cmd = 6;
		else
			R->cmd = 7;
	}
	R->ex = 0;
	R->ex = 1;
}

void ellipse(int x,int y, int xwidth, int ywidth, int draw , int filled )
{
	R->xcoord1 = x;
	R->ycoord1 = y;
	R->xcoord2 = xwidth;
	R->ycoord2 = ywidth;
	if( filled )
	{
		if( draw )
			R->cmd = 12;
		else
			R->cmd = 13;
		
	}else{
		if( draw )
			R->cmd = 10;
		else
			R->cmd = 11;
	}
	R->ex = 0;
	R->ex = 1;
}

void triangle(int rot, int x,int y, int xwidth, int ywidth, int draw , int filled )
{
	R->xcoord1 = x;
	R->ycoord1 = y;
	R->xcoord2 = xwidth;
	R->ycoord2 = ywidth;
	if( filled )
	{
		if( draw )
			R->cmd = 16;
		else
			R->cmd = 17;
		
	}else{
		if( draw )
			R->cmd = 14;
		else
			R->cmd = 15;
	}
	R->rot = rot;
	R->ex = 0;
	R->ex = 1;
}


void setcolour( int r, int g, int b )
{
	R->red = r;
	R->green = g;
	R->blue = b;
}

void	pixel( int x, int y, int draw )
{
	R->xcoord1 = x;
	R->ycoord1 = y;
	if( draw )
		R->cmd = 2;
	else
		R->cmd = 3;
	R->ex = 0;
	R->ex = 1;
}

void	line( int x1, int y1, int x2, int y2,int draw )
{
	R->xcoord1 = x1;
	R->ycoord1 = y1;
	R->xcoord2 = x2;
	R->ycoord2 = y2;
	if( draw )
		R->cmd = 4;
	else
		R->cmd = 5;
	R->ex = 0;
	R->ex = 1;
}

/* SysTick */
#define		STK_CTRL 	((volatile unsigned int *)(0xE000E010))  
#define		STK_LOAD 	((volatile unsigned int *)(0xE000E014))  
#define		STK_VAL 	((volatile unsigned int *)(0xE000E018))  

#define DELAY_VAL	168000/4-1
void delay( void )
{
	/* SystemCoreClock = 168000000 */
	*STK_CTRL = 0;
	*STK_LOAD = DELAY_VAL;
	*STK_VAL = 0;
	*STK_CTRL = 5;
	while( (*STK_CTRL & 0x10000 )== 0 );
	*STK_CTRL = 0;
}


//#define TEST_PIXEL
//#define TEST_LINE
//#define TEST_CIRCLE
//#define TEST_RECTANGLE
#define TEST_TRIANGLE




void main(void)
{
#ifdef TEST_PIXEL
    /* Roterande pixel. Rotationscentrum i display mitt */
	setbgnd( 255 );
	R->xcoord3 = 80;
	R->ycoord3 = 72;
	setcolour( 255, 0, 0 );
	
	while(1)
	{
		for( int i = 0; i<360;i = i + 1)
		{
			R->rot = i;
			pixel( 70, 72, 1 );
			delay();	
			pixel( 70, 72, 0 );
		}
	}
#endif

#ifdef TEST_LINE
/* Roterande linje. Rotationscentrum i linjens mitt */
	setbgnd( 200 );
	setcolour( 255, 0, 0 );
	R->xcoord3 = 80;
	R->ycoord3 = 72;
	
	while(1)
	{
		for( int i = 0; i<360;i = i + 1)
		{
			R->rot = i;
			line(20,72, 140, 72, 1 );
			delay();	
			line(20,72, 140, 72, 0 );
		}
	}
#endif



#ifdef TEST_CIRCLE
/* Liten roterande ellips. Rotationscentrum måste ligga utanför ellipsen */
	setbgnd( 200 );
	setcolour( 255, 0, 0 );
	R->xcoord3 = 40;
	R->ycoord3 = 72;
	setcolour( 255, 0, 0 );
	
	while(1)
	{
		for( int i = 0; i<360;i = i + 1)
		{
			R->rot = i;
			ellipse(80,72, 60, 30, 1 ,1);
			delay();	
			ellipse(80,72, 60, 30, 0, 1 );
		}
	}
#endif


#ifdef TEST_RECTANGLE
/* Roterande kvadrat. Rotationscentrum i kvadratens mitt */
	setbgnd( 200 );
	R->xcoord3 = 80;
	R->ycoord3 = 72;
	setcolour( 255, 0, 0 );
	while(1)
	{
		for( int i = 0; i<360;i = i + 1)
		{
			R->rot = i;
			rect(60,52, 40, 40, 1 , 1);
			delay();	
			rect(60,52, 40, 40, 0 , 1);
		}
	}
#endif
#ifdef TEST_TRIANGLE
/* Roterande triangel. Rotationscentrum i triangelns tyngdpunkt */
	setbgnd( 200 );
	R->xcoord3 = 80;
	R->ycoord3 = 72;
	setcolour( 255, 0, 0 );
	while(1)
	{
		for( int i = 0; i<360;i = i + 5)
		{
			triangle(i, 60,50, 120, 120, 1 , 0);
			delay();	
			triangle(i, 60,50, 120, 120, 0 , 0 );
		}
	}
#endif
}

